<?php
// old versions of php < 5.3.0 doesn't have E_DEPRECATED
if (defined('E_DEPRECATED'))
{
    error_reporting(E_ALL & ~E_NOTICE & ~E_DEPRECATED);
}
else
{
    error_reporting(E_ALL & ~E_NOTICE);
}

date_default_timezone_set('UTC');

// This is to resolve know bug in php built in server
// https://bugs.php.net/bug.php?id=66606
if (!isset($_SERVER['CONTENT_TYPE']))
{
    $_SERVER['CONTENT_TYPE'] = $_SERVER['HTTP_CONTENT_TYPE'];
}

require_once 'config/config.php';
require_once 'vendor/autoload.php';
require_once 'lib/oauth/OauthException.php';
require_once 'lib/oauth/OauthCfPdo.php';

require_once 'lib/DcSettings.php';
require_once 'lib/Tonic.php';
require_once 'lib/ProtectedResource.php';
require_once 'lib/Utils.php';
require_once 'lib/DefaultParameters.php';
require_once 'lib/git/Git.php';
require_once 'lib/git/CF_GitRepo.php';
require_once 'lib/SketchRequestCommand.php';
require_once 'lib/DcApi.php';
require_once 'lib/SketchApi.php';
require_once 'lib/SettingsHelper.php';
require_once 'lib/CfPdo.php';
require_once 'lib/CfdbPdo.php';
require_once 'lib/UsersList.php';
require_once 'lib/CFHostCount.php';
require_once 'lib/UsersListRequestStructure.php';
require_once 'lib/CfUsers.php';
require_once 'lib/CfRole.php';
require_once 'lib/CfEvent.php';
require_once 'lib/CfTasks.php';
require_once 'lib/health_diagnostics/autoload.php';
// load resources
require_once 'resource-v1/Host.php';
require_once 'resource-v1/Query.php';
require_once 'resource-v1/Settings.php';
require_once 'resource-v1/Status.php';
require_once 'resource-v1/Role.php';
require_once 'resource-v1/User.php';
require_once 'resource-v1/Sketch.php';
require_once 'resource-v1/DcWorkSpace.php';
require_once 'resource-v1/Oauth.php';
require_once 'resource-v1/Changes.php';
require_once 'resource-v1/Event.php';
require_once 'resource-v1/InternalTasks.php';
require_once 'resource-v1/HealthDiagnostic.php';
// load modules
require_once 'modules/inventory/autoload.php';

define('SELF', pathinfo(__FILE__, PATHINFO_BASENAME));
// Path to the front controller (this file)
define('FCPATH', str_replace(SELF, '', __FILE__));

$dcConfigFilePath = Utils::getDcConfigFilePath();
define('DCCONFIGPATH', $dcConfigFilePath);

$request = new Request(array(
    'baseUri' => '/api',
    'mimetypes' => array(
        'json-v1' => 'application/vnd.cfengine.enterprise-v1+json'
    )
));

try
{
    $format = DefaultParameters::format();
    if (!empty($request->accept))
    {
        $format = $request->mostAcceptable(array(
            DefaultParameters::format()
        ));
    }


    switch ($format)
    {
    case 'json-v1':

        break;

    default:
        throw new ResponseException("Unsupported format or version requested", Response::NOTACCEPTABLE);
    }

    DefaultParameters::check();

    $resource = $request->loadResource();

    if (isset($resource->username)) {
        apache_note('username', $resource->username);
    }

    $response = $resource->exec($request);
}
catch(OauthException $e)
{

    $response = new Response($request);
    $response->body = $e->getOauthBody();
    $response->code = $e->getOauthCode();

    foreach ($e->getOauthHttpHeaders() as $key => $value)
    {
        $response->addHeader($key, $value);
    }

}
catch(ResponseException $e)
{

    switch ($e->getCode())
    {
    case Response::UNAUTHORIZED:
        $response = $e->response($request);

        break;

    case Response::INTERNALSERVERERROR:
        $response = $e->response($request);

        break;

    default:
        $response = $e->response($request);
    }
}
catch(CFAPIException $e)
{
    $response = new Response($request);
    $response->body = $e->getMessage();
    $response->code = Utils::CFAPIErrorCodeToHttpCode($e->getCode());
}
catch(Exception $e)
{
    $response = new Response($request);
    $response->body = $e->getMessage();
    $response->code = Response::INTERNALSERVERERROR;
}

// Add newline to API error responses
if ($response->code >= 400) {
    $response->body = $response->body . PHP_EOL;
}

$response->addHeader('Content-type', 'application/vnd.cfengine.enterprise-v1+json');
$response->output();
return;
