<?php
abstract class DcWorkSpaceBase extends CfProtectedResource
{
  function __construct($parameters = array())
  {
    parent::__construct($parameters);


    $userSettings = new stdClass();
    $this->dcSettings = new DcSettings();

    $userSettings->username = $this->username;
    $userSettings->gitServer = $this->dcSettings->get_dc_staging_repository();
    $userSettings->gitBranch = $this->dcSettings->get_dc_git_working_branch();
    $this->workspaceApi = new DcApi($userSettings);
  }

  protected function _parseSettingText($paramFile)
  {
    $data = array();
    $content = file_get_contents($paramFile);
    $rows = explode("\n", $content);
    array_shift($rows);
    $pattern = '/^(\w+)="(.*)"$/i';


    foreach ($rows as $row)
    {
      if (preg_match($pattern, $row, $matches))
      {
        $key = $matches[1];
        $value = $matches[2];
        $data[$key] = $value;
      }
    }

    return $data;
  }

}
/**
 * @uri /dc/workspace
 */
class DcWorkSpace extends DcWorkSpaceBase
{
  /**
   * Creates workspace
   * @param  [type] $request [description]
   * @return [type]          [description]
   */
  function get($request)
  {
    $response = new Response($request);
    $this->workspaceApi->initializeWorkSpace($this->username);
    $this->workspaceApi->checkStagingRepository();
    $this->workspaceApi->checkUserRepository();
    $result = array(
      'workDir' => $this->dcSettings->get_dc_user_working_directory($this->username)
    );
    $response->body = utils::packageResult($result, 1, 1);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/workspace/commit
 */
class DcWorkSpaceCommit extends DcWorkSpaceBase
{
  function post($request)
  {
    $response = new Response($request);
    $data = json_decode($request->data, true);

    $requiredParameters = array('userEmail','message');
    Utils::checkRequiredParameters($data,$requiredParameters);

    $userEmail = $data['userEmail'];
    $message = $data['message'];

    $settings = array();
    $scriptsPath = $this->dcSettings->get_dc_update_script_path();
    $paramFile = $scriptsPath . DIRECTORY_SEPARATOR . 'params.sh';
    if (@file_exists($paramFile))
    {
      $settings = $this->_parseSettingText($paramFile);
    }

    $authorName = !empty($settings['GIT_AUTHOR']) ? $settings['GIT_AUTHOR'] : $this->username;
    $authorEmail = !empty($settings['GIT_EMAIL']) ? $settings['GIT_EMAIL']: $userEmail;

    $result = $this->workspaceApi->commit_push($message, $authorName, $authorEmail, $userEmail);
    $response->body = json_encode($result);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/workspace/reset
 */
class DcWorkSpaceReset extends DcWorkSpaceBase
{
  function post($request)
  {
    $response = new Response($request);
    $result = $this->workspaceApi->reset_and_pull();
    $response->body = json_encode($result);
    $response->code = Response::OK;

    return $response;
  }
}
/**
 * @uri /dc/workspace/settings
 */
class DcWorkSpaceSettings extends DcWorkSpaceBase
{

  private function _validateParameters($data)
  {
    $validationError = array();
    if (empty($data['gitServer']))
    {
      $error = new stdClass();
      $error->field = 'gitServer';
      $error->message = 'Git server not set';
      array_push($validationError, $error);
    }

    if (empty($data['gitEmail']))
    {
      $error = new stdClass();
      $error->field = 'gitEmail';
      $error->message = 'Committer email not set';
      array_push($validationError, $error);
    }
    if (empty($data['gitAuthor']))
    {
      $error = new stdClass();
      $error->field = 'gitAuthor';
      $error->message = 'Committer name not set';
      array_push($validationError, $error);
    }

    if (empty($data['gitPrivateKey']))
    {
      $error = new stdClass();
      $error->field = 'gitPrivateKey';
      $error->message = 'Private key not set';
      array_push($validationError, $error);
    }

    return empty($validationError) ? true : $validationError;
  }
  function get($request)
  {
    $response = new Response($request);
    $scriptsPath = $this->dcSettings->get_dc_update_script_path();
    $paramFile = $scriptsPath . DIRECTORY_SEPARATOR . 'params.sh';
    if (@file_exists($paramFile))
    {
      $data = $this->_parseSettingText($paramFile);
      $response->body = utils::packageResult($data, 1, 1);
      $response->code = Response::OK;
    }
    else
    {
      $data = array(
        'message' => "No configuration found at path $paramFile"
      );
      $response->code = Response::NOTFOUND;
      $response->body = json_encode($data);
    }

    return $response;
  }
  function post($request)
  {
    $response = new Response($request);
    $data = (array) Utils::getValidJsonData($request->data);
    $validation = $this->_validateParameters($data);
    if ($validation === true)
    {

      try
      {
        $filename = $data['gitPrivateKey'];
        $destination = $this->dcSettings->get_dc_user_ssh_key_directory($this->username);
        $uploadName = $this->dcSettings->get_dc_user_ssh_key_name();
        $fullPath = $destination . DIRECTORY_SEPARATOR . $uploadName;
        if (copy($filename, $fullPath) === true)
        {
          chmod($fullPath, 0600);
          $result = $this->workspaceApi->gitInitStagingFromRemote($data);
          $response->body = json_encode($data);
          $response->code = Response::OK;
        }
        else
        {

          throw new Exception("Error uploading file Request", 1);
        }
      }
      catch(Exception $e)
      {
        $data = $e->getMessage();
        $response->code = Response::INTERNALSERVERERROR;
        $response->body = json_encode($data);
      }
    }
    else
    {
      $response->code = Response::BADREQUEST;
      $messages = array();
      foreach ($validation as $error)
      {
        $messages[] = $error->message;
      }
      $data = implode("\n", $messages);
      $response->body = json_encode($data);
    }

    return $response;
  }
  function delete($request)
  {
    $response = new Response($request);
    $keyfile = $this->dcSettings->get_dc_user_ssh_key_directory($this->username) . DIRECTORY_SEPARATOR . $this->dcSettings->get_dc_user_ssh_key_name();
    @unlink($keyfile);
    $scriptsPath = $this->dcSettings->get_dc_update_script_path();
    Utils::delete_git_update_scripts($scriptsPath);
    $response->body = json_encode($data);
    $response->code = Response::OK;

    return $response;
  }
}
?>
