<?php

/**
 * @uri /host/([^\/]+)/vital/([^\/]+) 4
 */
class HostVital extends CfProtectedResource
{
    function get($request, $host_id, $vital_id)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_host_vital_get($user,
                $host_id,
                $vital_id,
                DefaultParameters::from(),
                DefaultParameters::to());
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /host/(.+)/context 3
 */
class HostContextList extends CfProtectedResource
{
    function get($request, $host_id)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_host_context_list($user, $host_id);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /host/(.+)/vital 2
 */
class HostVitalList extends CfProtectedResource
{
    function get($request, $host_id)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_host_vital_list($user, $host_id);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /host/:host_id 1
 */
class Host extends CfProtectedResource
{
    function get($request, $host_id)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_host_get($user, $host_id);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }

    function delete($request, $host_id)
    {
        $user = $this->username;
        $response = new Response($request);

        $cfUser = new CfUsers($user);
        /**
         * If a user has admin role then do not check host presence.
         * Otherwise there is no way to delete not reported hosts, because host won't be found
         * due to database RBAC.
         **/
        $shouldDelete = $cfUser->isAdminRole() ? true : cfapi_host_get($user, $host_id);
        if ($shouldDelete) {
            if (cfapi_host_delete($user, $host_id)) {
                $response->code = Response::ACCEPTED;
            } else {
                $response->code = Response::INTERNALSERVERERROR;
            }
        } else {
            $response->code = Response::NOTFOUND;
        }

        return $response;
    }
}

/**
 * @uri /host 0
 */
class HostList extends CfProtectedResource
{
    function get($request)
    {
        $user = $this->username;

        $context_include = array();
        if (Utils::queryParam('context-include'))
        {
            $context_include = [Utils::queryParam('context-include')];
        }

        $context_exclude = array();
        if (Utils::queryParam('context-exclude'))
        {
            $context_exclude = [Utils::queryParam('context-exclude')];
        }

        $response = new Response($request);
        $response->body = cfapi_host_list($user,
                $context_include,
                $context_exclude,
                DefaultParameters::page(), DefaultParameters::count());
        $response->code = Response::OK;

        return $response;
    }

    function post($request)
    {
        $user = $this->username;
        $data = Utils::getValidJsonData($request->data);

        $context_include = array();
        if (isset($data->{"context-include"}) && !empty($data->{"context-include"})) {
            $context_include = [$data->{"context-include"}];
        }

        $context_exclude = array();
        if (isset($data->{"context-exclude"}) && !empty($data->{"context-exclude"})) {
            $context_exclude = [$data->{"context-exclude"}];
        }

        $response = new Response($request);
        $response->body = cfapi_host_list($user,
            $context_include,
            $context_exclude,
            DefaultParameters::page(), DefaultParameters::count());
        $response->code = Response::OK;

        return $response;
    }
}

/**
 * @uri /host-count 0
 */
class HostsCount extends CfProtectedResource
{
    function post($request)
    {
        $data = Utils::getValidJsonData($request->data);

        $this->validate($data);
        $model = new CFHostCount(
            $data->from,
            $data->to,
            $data->period
        );

        $model->setRestClient(new PestHttpClient(API_URL));

        $data = $model->getCountByPeriod();


        $response = new Response($request);
        $response->body = json_encode($data, JSON_PRETTY_PRINT);
        $response->code = Response::OK;

        return $response;
    }

    private function validate($data){

        $errors = [];
        if ($data->period === NULL || !isset(CFHostCount::$periodsMap[strtolower($data->period)])) {
            $errors[] = 'Selected period is wrong. Period can be: ' . implode(', ', array_keys(CFHostCount::$periodsMap));
        }

        if($data->from === NULL){
            $errors[] = 'From cannot be blank';
        }

        if($data->to === NULL){
            $errors[] = 'To cannot be blank';
        }

        if(!empty($errors)){
            throw new Exception(implode("\n", $errors));
        }

    }
}
