<?php

/**
 * @uri /role/:rolename 0
 */
class Role extends CfProtectedResource
{
    function get($request, $rolename)
    {
        $user = $this->username;

        $response = new Response($request);
        $payload = cfapi_role_get($user, $rolename);
        if ($payload)
        {
            $response->code = Response::OK;
            $response->body = $payload;
        }
        else
        {
            $response->code = Response::NOTFOUND;
        }


        return $response;
    }

    function put($request, $rolename)
    {
        $user = $this->username;

        $data =  Utils::getValidJsonData($request->data);

        $response = new Response($request);
        if (cfapi_role_put($user, $rolename, $data->description,
                $data->includeContext, $data->excludeContext,
                $data->includeSketches))
        {
            $response->code = Response::CREATED;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }

    function post($request, $rolename)
    {
        $user = $this->username;
        $response = new Response($request);

        $data =  Utils::getValidJsonData($request->data);

        if (cfapi_role_post($user, $rolename, $data->description,
                $data->includeContext, $data->excludeContext,
                $data->includeSketches))
        {
            $response->code = Response::NOCONTENT;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }

    function delete($request, $rolename)
    {
        $user = $this->username;

        $response = new Response($request);

        if (cfapi_role_delete($user, $rolename))
        {
            $response->code = Response::NOCONTENT;
        }
        else
        {
            $response->code = Response::INTERNALSERVERERROR;
        }

        return $response;
    }
}

/**
 * @uri /role 1
 */
class RoleList extends CfProtectedResource
{
    function get($request)
    {
        $user = $this->username;

        $response = new Response($request);
        $response->body = cfapi_role_list($user);
        $response->code = Response::OK;

        return $response;
    }
}

/**
 * @uri /external_role 0
 */
include_once __DIR__ . '/../lib/CfExternalRoles.php';
class ExternalRoles extends CfProtectedResource
{
    private $cfExternalRoles;

    public function __construct(array $parameters)
    {
        $this->cfExternalRoles = new CfExternalRoles(CfSettings::getInstance()->getConnection());
        parent::__construct($parameters);
    }

    /**
     * @param $request
     * @return Response
     */
    function get($request)
    {
        $data  =  $_GET;
        $response = new Response($request);
        $response->body = json_encode($this->cfExternalRoles->getExternalRoleByInternal($data['internal_role']));
        $response->code = Response::OK;

        return $response;
    }

    /**
     * @param $request
     * @return Response
     * @throws Exception
     * @throws ResponseException
     */
    function post($request)
    {

        $data  =  Utils::getValidJsonData($request->data);
        $this->validateData($data);

        $res = $this->cfExternalRoles->addRoleMap($data->external_role, $data->internal_role);
        $response = new Response($request);
        $response->body = json_encode($res);
        $response->code = Response::OK;

        return $response;
    }

    /**
     * @param $request
     * @return Response
     * @throws Exception
     * @throws ResponseException
     */
    function patch($request)
    {

        $data  =  Utils::getValidJsonData($request->data);
        $this->validateData($data);

        $res = $this->cfExternalRoles->editRoleMap($data->external_role, $data->internal_role);
        $response = new Response($request);
        $response->body = json_encode($res);
        $response->code = Response::OK;

        return $response;
    }

    /**
     * @param $request
     * @return Response
     * @throws Exception
     * @throws ResponseException
     */
    function delete($request)
    {

        $data  =  $_GET;
        $res = $this->cfExternalRoles->deleteRoleMap($data['internal_role']);
        $response = new Response($request);
        $response->body = json_encode($res);
        $response->code = Response::OK;

        return $response;
    }

    /**
     * @param $data
     * @throws Exception
     */
    private function validateData($data){

        if(!isset($data->external_role)){
            throw new Exception('External_role field is required.');
        }

        if(!isset($data->internal_role)){
            throw new Exception('Internal_role field is required.');
        }
    }

}

/**
 * @uri /default_role
 */
class DefaultRole extends CfProtectedResource
{
    /**
     * @var CfRole
     */
    private $roleModel;

    /**
     * DefaultRole constructor.
     * @param array $parameters
     */
    public function __construct(array $parameters)
    {
        $this->roleModel = new CfRole();
        parent::__construct($parameters);
    }

    function get($request)
    {
        $this->checkForAdminRole(new CfUsers($this->username));
        $response = new Response($request);
        $response->body = json_encode($this->roleModel->getDefaultRole());
        $response->code = Response::OK;

        return $response;
    }

    function post($request)
    {
        $this->checkForAdminRole(new CfUsers($this->username));
        $data =  Utils::getValidJsonData($request->data);
        $response = new Response($request);
        $response->body = json_encode($this->roleModel->setDefaultRole($data->name));
        $response->code = Response::OK;

        return $response;
    }
}