<?php

//class autocomplete extends Cf_Controller

require_once APPPATH . '/libraries/Cf_REST_Controller.php';

class autocomplete extends Cf_REST_Controller
{

    protected $username = '';

    function __construct()
    {
        parent::__construct();
        $this->load->model(array('advancedreports_model'));
        $this->advancedreports_model->setRestClient($this->getRestClient());
        $this->username = $this->session->userdata('username');
    }

    function index()
    {
        $this->respond_forbidden();
    }

    /**
     * Return field data for autocomplete list
     *
     * Request body contain:
     *  $fieldName - type of variable - package name, version number...
     *  $value - value from input field
     *  $parentFieldName  - type(sqlfield name) of parent element - 'patchname'
     *  $parentValue - value of parent element - in case list depends on parent value, for example - package version depends on package name
     */
    function load_post()
    {
        $data = json_decode($this->request->body, true);

        $fieldName = $data['type'];
        $value     = $data['value'];

        $parentFieldName = isset($data['parentType'])  ?  $data['parentType']  : '';
        $parentValue     = isset($data['parentValue']) ?  $data['parentValue'] : '';

        try {
            $result = $this->advancedreports_model->getAutocompleteFieldData($this->username, $fieldName, $value, $parentFieldName, $parentValue);
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('error', 'load_get(): Unable to get autocomplete list for '. $fieldName .', error: ' . $message);
            respond_internal_error($message);
            return;
        }

        respond_ok(json_encode($result));
    }

    /**
     * Return all available sketches
     *
     * @param <string> $formValue - string from input
     * @return <json>
     */
    function sketch_get($formValue)
    {

        // TODO
        // add check if DC app is available - and return error if not


        // TODO
        // move to model

        $jsonBody = $this->getRestClient()->get('/dc/sketch');

        $sketchesRaw = json_decode($jsonBody, true);

        if (empty($sketchesRaw['data'])) {
            respond_ok();
            return;
        }


        $sketchesNames = array();

        foreach ($sketchesRaw['data'] as $sketchName => $sketchData) {
            $sketchesNames[] = $sketchName;
        }

        // filter array data similar to SQL LIKE  %$value%
        $result = array_filter($sketchesNames, function ($item) use ($formValue) {
            if (stripos($item, $formValue) !== false) {
                return true;
            }
            return false;
        });

        respond_ok(json_encode(array_values($result)));
    }

    /**
     * Return info about ACTIVE sketches - sketches which have at least one activation
     *
     * @param <string> $formValue
     * @return <json>
     */
    function activesketch_get($formValue)
    {

        // TODO
        // add check if DC app is available - and return error if not


        // TODO
        // move to model
        try {
            $jsonBody = $this->getRestClient()->get('/dc/activation');
        }
        catch(Exception $e)
        {
            $message = $e->getMessage();
            log_message('error', 'activesketch_get(): Unable to get activations list, error: ' . $message);
            respond_internal_error($message);
            return;
        }

        $sketchesRaw = json_decode($jsonBody, true);

        if (empty($sketchesRaw['data'])) {
            respond_ok();
            return;
        }


        $sketchesNames = array();

        foreach ($sketchesRaw['data'] as $sketchName => $sketchData) {
            $sketchesNames[] = $sketchName;
        }

        // filter array data similar to SQL LIKE  %$value%
        $result = array_filter($sketchesNames, function ($item) use ($formValue) {
            if (stripos($item, $formValue) !== false) {
                return true;
            }
            return false;
        });

        respond_ok(json_encode(array_values($result)));
    }


    /**
     * List of activations for one check (by name - $sketchName)
     * @param <string> $sketchName
     * @param <string> $formValue - substring to filter

     * @return <json>
     */
    function sketchactivation_get($sketchName, $formValue)
    {
        // TODO
        // add check if DC app is available - and return error if not

        // TODO
        // move to model

        $jsonBody = $this->getRestClient()->get('/dc/activation?sketch='.$sketchName.'&details=1');

        $activationsRaw = json_decode($jsonBody, true);

        if (empty($activationsRaw['data'])) {
            respond_ok();
            return;
        }

        $activations = array();

        foreach ($activationsRaw['data'] as $item => $info) {
            $activations[] = $info['identifier'];
        }

        // filter array data similar to SQL LIKE  %$value%
        $result = array_filter($activations, function ($item) use ($formValue) {
            if (stripos($item, $formValue) !== false) {
                return true;
            }
            return false;
        });

        respond_ok(json_encode(array_values($result)));
    }

    /**
     * Return active sketches with activations
     */
    function sketchesandactivations_get()
    {
        $this->load->model('designcenter/dc_rest_model');

        $result = array();

        try
        {
            $DC_DATA = $this->dc_rest_model->getActivations();

            foreach ($DC_DATA as $sketch => $info)
            {
                foreach ($info as $activationItem => $activation)
                {
                    $result[$sketch][] = $activation;
                }
            }

              respond_ok(json_encode($result));
        }
        catch (Exception $e)
        {
            $message = $e->getMessage();
            log_message('error', 'sketchesandactivations_get(): Unable to get sketch and activations list, error: ' . $message);
            respond_internal_error($message);
            return;
        }

        if (empty($result))
        {
            respond_ok();
            return;
        }
    }

}
?>