<?php
require_once 'Pest.php';
/**
 * Small Pest addition by Egbert Teeselink (http://www.github.com/eteeselink)
 *
 * Pest is a REST client for PHP.
 * PestJSON adds JSON-specific functionality to Pest, automatically converting
 * JSON data resturned from REST services into PHP arrays and vice versa.
 *
 * In other words, while Pest's get/post/put/delete calls return raw strings,
 * PestJSON return (associative) arrays.
 *
 * In case of >= 400 status codes, an exception is thrown with $e->getMessage()
 * containing the error message that the server produced. User code will have to
 * json_decode() that manually, if applicable, because the PHP Exception base
 * class does not accept arrays for the exception message and some JSON/REST servers
 * do not produce nice JSON
 *
 * See http://github.com/educoder/pest for details.
 *
 * This code is licensed for use, modification, and distribution
 * under the terms of the MIT License (see http://en.wikipedia.org/wiki/MIT_License)
 */
class pest_json extends Pest
{
    var $token = null;

    public function post($url, $data = array() , $headers = array())
    {
        $headers[] = 'Content-Type: application/json';
        return parent::post($url, json_encode($data) , $headers);
    }

    public function post_multipart($url, $data, $headers = array())
    {
        return parent::post($url, $data, $headers);
    }

    public function put($url, $data, $headers = array())
    {
        $headers[] = 'Content-Type: application/json';
        return parent::put($url, json_encode($data) , $headers);
    }

    protected function prepRequest($opts, $url)
    {
        $opts[CURLOPT_HTTPHEADER][] = 'Accept: application/vnd.cfengine.nova-v1+json';

        if ($this->token)
        {
            $opts[CURLOPT_HTTPHEADER][] = 'Authorization: Bearer ' . $this->token;
        }

        return parent::prepRequest($opts, $url);
    }

    public function __construct($param)
    {
        parent::__construct($param['base_url']);
    }

    public function processError($body)
    {
        if ($this->lastStatus() === 0)
        {
            return "Error encountered from API server: $body";
        }
        return $body;
    }

    public function setupOauthAuth($token = '')
    {
        if ($token)
        {
            $this->token = $token;
        }
    }
}
