<?php

class License_model extends Cf_Model
{

    protected $rest = '';

    public function __construct()
    {
        parent::__construct();
    }

    /**
     *
     * @return Array of information
     */
    public function getLicenseDetails()
    {
        try
        {
            $apiInfo = $this->rest->get('/');
            $data = $this->checkData($apiInfo);
            if (is_array($data) && $this->hasErrors() == 0)
            {
        
                if (isset($data['data'][0]['license'])) {
                    return $data['data'][0]['license'];
                }

            }
            else
            {
                throw new Exception($this->getErrorsString());
            }
        }
        catch (Exception $e)
        {
            log_message('error', $e->getMessage() . " " . $e->getFile() . " line:" . $e->getLine());
            throw $e;
        }
    }

    /**
     * get number of days between 2 time stamps
     * @param type $start_ts
     * @param type $end_ts
     * @return int number of days
     */
    function _getDays($start_ts, $end_ts)
    {
        $diff = $end_ts - $start_ts;
        return floor($diff / 86400);
    }

    /**
     * Calculate days left and other infiormation about license.
     * if licenseType = "Enterprise Free" we won't show expire date
     *
     * @param array $licenseDetails
     * @return array
     */
    public function caclulateLicenseDays($licenseDetails)
    {
        if (isset($licenseDetails['licenseType']) && !empty($licenseDetails['licenseType']) && $licenseDetails['licenseType'] != 'Enterprise Free')
        {
            $expirydate = intval($licenseDetails['expires']);
            $startDate = intval($licenseDetails['installTime']);

            $totaldays = $this->_getDays($startDate, $expirydate);
            if ($totaldays > 0)
            {
                $dayspassed = $this->_getDays($startDate, time());
                $pbarvalue = floor(($dayspassed / $totaldays) * 100);
                $daysleft = $totaldays - $dayspassed;
            }
            else
            {
                $pbarvalue = 100;
                $daysleft = $totaldays;
            }

            $licenseDetails['pbarvalue'] = $pbarvalue;
            $licenseDetails['daysleft'] = $daysleft;

            $licenseDetails['started'] = date('D F d h:m:s Y', $licenseDetails['installTime']);
            $licenseDetails['expiry'] = date('D F d h:m:s Y', $licenseDetails['expires']);
        }
        return $licenseDetails;
    }

}

?>