<?php

class Designcenter extends Cf_Controller
{

    function __construct()
    {
        parent::__construct();
        setCurrentApplication('designcenter');
    }

    function index()
    {
        $requiredcss = array(
            array('angularjs-ui/angular-ui.css'),
            array('jsoneditor.css'),
            array('chosen.min.css'),
            array('chosen-spinner.css')
        );

        $this->carabiner->css($requiredcss);

        $jsIE = array('libs/angularjs-ui/angular-ui-ieshiv.min.js');
        $this->carabiner->group('iefix', array('js' => $jsIE));

        $requiredAngularjs = array(
            array('libs/underscorejs/underscore.js'),
            array('libs/angularjs/angular.1.2.4.min.js'),
            array('libs/angularjs/angular-route.min.js'),
            array('libs/angularjs-ui/angular-ui.min.js'),
            array('libs/angularjs/angular-cookies.js'),

            // services
            array('libs/angularjs-ui/ui-bootstrap-tpls-0.6.0.min.js'),
            array('libs/class.js'),
            array('libs/namespace.js'),

            array('chosen.jquery.min.js'),
            array('angular-chosen.js'),
        );

        $requiredJsonEditor = array(
            array('libs/jsoneditor-2.3.3/jsoneditor-min.js'),
            array('libs/jsoneditor-2.3.3/lib/ace/ace.js'),
            array('libs/jsoneditor-2.3.3/lib/ace/mode-json.js'),
            array('libs/jsoneditor-2.3.3/lib/ace/theme-textmate.js'),
            array('libs/jsoneditor-2.3.3/lib/ace/theme-jsoneditor.js')
        );

        $requiredjs = array(// project
            array('libs/baseclasses/BaseController.js'),
            array('libs/baseclasses/EventDispatcher.js'),
            array('libs/baseclasses/Notifications.js'),

            array('designcenter/controllers/indexController.js'),
            array('designcenter/controllers/sketchController.js'),
            array('designcenter/controllers/sketchConfigController.js'),
            array('designcenter/directives/sketchCategoryDirective.js'),
            array('designcenter/directives/environmentSelectionDirective.js'),
            array('designcenter/directives/sketchValidationDirective.js'),
            array('designcenter/directives/csvListDirective.js'),
            array('designcenter/directives/arrayInputType.js'),
            array('designcenter/directives/jsonInputType.js'),
            array('designcenter/directives/index.js'),
            array('designcenter/models/sketchModel.js'),
            array('designcenter/models/sketchFormInputModel.js'),
            array('designcenter/models/index.js'),
            array('designcenter/services/sketchService.js'),
            array('designcenter/services/dashboardService.js'),

            array('designcenter/services/naviTreeService.js'),
            array('designcenter/filters/datefilter.js'),
            array('designcenter/app.js'),
            // notification should be loaded after app
            array('designcenter/services/notificationService.js'),
        );

        $this->carabiner->js($requiredJsonEditor, $requiredJsonEditor, $combine = false, $minify = false);
        $this->carabiner->js($requiredAngularjs, $requiredAngularjs, $combine = true, $minify = false);
        $this->carabiner->js($requiredjs, $requiredjs, $combine = true, $minify = true, $group = 'designCenter');

        $data = array('title' => 'Design Center');



        $this->config->load('designcenter');
        $this->load->helper('designcenter/dc_path');
        $this->load->helper(array('url', 'cf_directory_helper'));
        $this->load->model('designcenter/dc_rest_model');

        $username = $this->session->userdata('username');

        $data['ALLOW_RUNAGENT'] = false;
        $data['ALLOW_COMMIT'] = false;
        $data['isAdmin'] = false;

        $user_role = $this->session->userdata('roles');
        if (empty($user_role))
        {
            $user_role = array();
        }
        if (in_array('admin',$user_role) === true)
        {
            $data['isAdmin'] = true;
        }

        try
        {
           $this->dc_rest_model->getWorkSpace();
        }
        catch (Exception $e)
        {
            $error = $e->getMessage();
            $data['errorType'] = 'GENERIC';
            $data['error'] = $error;
            $this->template->load('template', 'designcenter/giterror', $data);
            return;
        }

        // check if the user has cf_remoteagent role
        if (in_array('cf_remoteagent', $user_role) === true)
        {
           $data['ALLOW_RUNAGENT'] = true;
        }



        $this->template->load('template', 'designcenter/index', $data);
    }

    function clonestage()
    {
        $this->config->load('designcenter');
        $this->load->helper('designcenter/dc_path');
        $this->load->helper(array('url', 'cf_directory_helper'));
        $this->load->library('form_validation');
        $this->load->helper('form');
        $feedback = array();




        if ($this->input->post('update'))
        {
            $feedback = $this->_handleFormUploadForCloningStage();
            if ($feedback === true)
            {
                redirect('designcenter/clonestage');
            }
        }
        $data = array('title' => 'Design Center','message' => $feedback);


        $this->template->load('template', 'designcenter/clonestage', $data);

    }

    function _handleFormUploadForCloningStage()
    {
        $this->load->library('git_service', array());
        $fileUploaded = empty($_FILES['cloneFile']['name']) ? false : true;
        if (!$fileUploaded)
        {
            // if no file is supplied then path must be supplied
            $this->form_validation->set_rules('clonePath', 'clone path', 'xss_clean|trim|required');
        }
        else
        {
             $this->form_validation->set_rules('clonePath', 'clone path', 'xss_clean|trim');
        }


        $feedback = array();
        if ($this->form_validation->run() === TRUE)
        {
            $username = $this->session->userdata('username');
            try
            {
                if ($fileUploaded)
                {
                    $extractedPath = $this->_unzipUploadedFile();
                    if ($extractedPath)
                    {
                        $this->git_service->gitInitStagingFromPath($extractedPath,$username);
                        deleteAll($extractedPath);
                    }
                }
                else
                {
                    $clonePath = $this->input->post('clonePath', true);
                    $this->git_service->gitInitStagingFromPath($clonePath,$username);
                }
                $feedback = array('type' => 'alert-success', 'content' =>"Creating staging repository successful.");
            }
            catch (Exception $e)
            {
                $feedback = array('type' => 'alert-error', 'content' => $e->getMessage());
            }
        }
        else
        {
            $feedback = validation_errors();
        }

        return $feedback;
    }

    function _unzipUploadedFile()
    {
        $config['upload_path'] = get_tmpdir();
        $config['allowed_types'] = 'zip';
        $config['overwrite'] = true;
        $extractPath = get_tmpdir().DIRECTORY_SEPARATOR.'policyextract';
        $this->load->library('upload', $config);
        if (!$this->upload->do_upload('cloneFile'))
        {
           throw new Exception($this->upload->display_errors());
        }
        else
        {
            $data = array('upload_data' => $this->upload->data());
            $zip = new ZipArchive;
            $file = $data['upload_data']['full_path'];
            if ($zip->open($file) === TRUE)
            {
                chmod($file,0644);
                @mkdir($extractPath,0777,true);
                $status = $zip->extractTo($extractPath);
                if ($status)
                {
                    $zip->close();
                    unlink($file);
                    return $extractPath;
                }
                else
                {
                    throw new Exception("Cannot extract the zip file.");
                }
            }
            else
            {
                throw new Exception("Cannot open compressed file");
            }
        }
        return false;
    }

    function partials($partial)
    {
        $view = 'partials/' . $partial . '.php';
        $this->load->view($view);
    }

}

?>