<?php

class cf_sketch_request
{

    public $version = '0.0.1';
    public $request = array();

    public function setRequest($requestCommandArray)
    {
        $this->request = $requestCommandArray;
    }

    public function getCommand()
    {

        $commandObj = new stdClass;
        $commandObj->dc_api_version = "0.0.1";
        $commandObj->request = $this->request;
        return $commandObj;
    }

    function searchAll($filter = true)
    {

        $listCommand = new stdClass();
        $listCommand->describe = true;
        if (is_array($filter) && !empty($filter))
        {
            $filter = $filter;
        }
        else
        {
            $filter = true;
        }
        $listCommand->search = $filter;
        $this->setRequest($listCommand);
    }

    function getSketchInformation($sketch)
    {
        $command = new stdClass();
        $command->describe = true;
        $command->search = array(array("name", "equals", $sketch));
        $this->setRequest($command);
    }

    function getActivations()
    {
        $command = new stdClass();
        $command->activations = true;
        $this->setRequest($command);
    }

    function activateSketch($sketchName, $target, $environment, $bundle, $params, $metaData = array())
    {
        $command = new stdClass();
        $activateParams[$sketchName] = array(
            'target' => $target,
            'environment' => $environment,
            'params' => $params,
            'metadata' => $metaData,
            'bundle' => $bundle,
            'identifier' => $metaData['identifier']
        );
        $command->activate = $activateParams;
        $this->setRequest($command);
    }

    function install($sketchName, $source, $target, $force)
    {

        $command = new stdClass();
        $installParams = array(
            array(
                'sketch' => $sketchName,
                'target' => $target,
                'force' => $force,
                'source' => $source
        ));
        $command->install = $installParams;
        $this->setRequest($command);
    }

    function uninstall($sketchName,$target)
    {
        $command = new stdClass();
        $uninstallParams = array(
            array(
                'sketch' => $sketchName,
                'target' => $target
        ));
        $command->uninstall = $uninstallParams;
        $this->setRequest($command);
    }

    function defineParam($sketchName, $paramName, $params = array())
    {
        if (empty($params))
        {
            $params = new stdClass(); // api expects object, assoc array are automatically objects
        }
        $command = new stdClass();
        $varName = $paramName;
        $defineParam[$varName] = array(
            $sketchName => $params
        );
        $command->define = $defineParam;
        $this->setRequest($command);
    }

    function undefineParam($paramNameArray = array())
    {

        $command = new stdClass();
        $command->undefine = $paramNameArray;
        $this->setRequest($command);
    }

    function getDefinitions()
    {
        $command = new stdClass();
        $command->definitions = true;
        $this->setRequest($command);
    }

    function defineEnvironment($envName, $environment=array())
    {

        $command = new stdClass();
        $defineParam[$envName] = array(
            'activated' => array('include' => $environment, 'exclude' => array()),
            'test' => false,
            'verbose' => false
        );
        $command->define_environment = $defineParam;
        $this->setRequest($command);
    }

    function undefineEnvironment($environments = array())
    {

        $command = new stdClass();
        $command->undefine_environment = $environments;
        $this->setRequest($command);
    }


    function getEnvironments()
    {
        $command = new stdClass();
        $command->environments = true;
        $this->setRequest($command);
    }

    function regenerate()
    {
        $command = new stdClass();
        $command->regenerate = true;
        $this->setRequest($command);
    }

    function getValidations()
    {
        $command = new stdClass();
        $command->validations = true;
        $this->setRequest($command);
    }

    function validate($validationType, $validationData)
    {
        $command = new stdClass();
        $validationArray = array('validation' => $validationType, 'data' => $validationData);
        $command->validate = $validationArray;
        $this->setRequest($command);
    }

    function deactivate($id)
    {
        $command = new stdClass();
        $command->deactivate = $id;
        $this->setRequest($command);
    }

    function generateJson()
    {
        $command = $this->getCommand();
        return json_encode($command);
    }

}

class cf_sketch_api
{

    protected static $bin = '/usr/bin/perl';
    protected $cf_sketch_api_executable = 'cf-dc-api.pl';
    protected $cf_sketch_api_script_path = '';
    protected $cf_sketch_api_config_file = 'api-config.json';
    public $clonePath = '';
    public $username = '';

    function __construct($data = array())
    {

        if (!empty($data))
        {
            $this->initialize($data);
        }
    }

    function initialize($config = array())
    {
        foreach ($config as $key => $val)
        {
            if (isset($this->$key))
            {
                $method = 'set_' . $key;

                if (method_exists($this, $method))
                {
                    $this->$method($val);
                }
                else
                {
                    $this->$key = $val;
                }
            }
        }
    }

    public static function setBinPath($path)
    {
        self::$bin = $path;
    }

    public static function getBin()
    {
        return self::$bin;
    }

    protected function run_command($command)
    {
        log_message('debug', $command);
        $descriptorspec = array(
            1 => array('pipe', 'w'),
            2 => array('pipe', 'a'),
        );
        $pipes = array();
        $resource = proc_open($command, $descriptorspec, $pipes, $this->cf_sketch_api_script_path);

        $stdout = stream_get_contents($pipes[1]);
        $stderr = stream_get_contents($pipes[2]);


        foreach ($pipes as $pipe)
        {
            fclose($pipe);
        }

        $status = trim(proc_close($resource));

        if ($status && !empty($stderr))
        {
            log_message('error', $command);
            throw new Exception($stderr);
        }
        $this->parse_api_response($stdout, $stderr, $command);
        log_message('debug', 'STDOUT --> ' . $stdout);
        log_message('debug', 'STDERR --> ' . $stderr);
        return $stdout;
    }

    private function parse_api_response($response, $stderr, $command = '')
    {
        $data = json_decode($response, true);
        if (!empty($data))
        {
            if (key_exists('api_ok', $data) !== false)
            {
                if ($data['api_ok']['success'] != true)
                {
                    $message = implode(',', $data['api_ok']['errors']);
                    throw new Exception($message);
                }
                else
                {
                    return;
                }
            }
            else if (key_exists('api_error', $data) !== false)
            {
                $message = $data['api_error'];
                $message .= $command;
                throw new Exception($message);
            }
        }
        throw new Exception("undefinded exception : $response with command :: " . $command);
    }

    /**
     *
     * @param JSON $command
     * @return type
     */
    public function run($request)
    {
        $json = $request->generateJson();
        $command = sprintf("%s %s %s '%s'", self::getBin(), $this->cf_sketch_api_executable, $this->cf_sketch_api_config_file, $json);
        return $this->run_command($command);
    }

    public function getAllSketch($filter = null)
    {

        $request = new cf_sketch_request();
        $request->searchAll($filter);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $data = $apiArray['api_ok']['data']['search'];
        $sketches = array();
        foreach ($data as $key=>$sources)
        {
            $sketches = array_merge($sketches,$sources);
        }
        return $sketches;
    }

    public function getSketchInformation($sketch)
    {
        $request = new cf_sketch_request();
        $request->getSketchInformation($sketch);
        $response = $this->run($request);
        $response = json_decode($response, true);
        $data = $response['api_ok']['data']['search'];
        $respond = array();
        foreach ($data as $key=>$sketchList)
        {
            if (array_key_exists($sketch,$sketchList) !== false)
            {
                $respond = $sketchList[$sketch];
            }
        }
        return $respond;
    }

    public function getSketchSourcePath($sketch)
    {
        $request = new cf_sketch_request();
        $request->getSketchInformation($sketch);
        $response = $this->run($request);
        $response = json_decode($response, true);
        $data = $response['api_ok']['data']['search'];
        $respond = "";
        foreach ($data as $key=>$sketchList)
        {
            if (array_key_exists($sketch,$sketchList) !== false)
            {
                $respond = $key;
            }
        }
        return $respond;
    }

    public function getActivations($sketchName = null)
    {
        $request = new cf_sketch_request();
        $request->getActivations();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $data = $apiArray['api_ok']['data']['activations'];
        if ($sketchName)
        {
            if (array_key_exists($sketchName, $data))
            {
                $data = $data[$sketchName];
            }
            else
            {
                $data = array();
            }
        }
        return $data;
    }

    public function activate($sketchName, $target, $environment, $bundle, $params, $metadata = array())
    {
        $request = new cf_sketch_request();
        $request->activateSketch($sketchName, $target, $environment, $bundle, array($params), $metadata);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['activate'];
        return $respond;
    }

    public function install($sketchName, $source, $target, $force)
    {
        $request = new cf_sketch_request();
        $request->install($sketchName, $source, $target, $force = true);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['install'];
        return $respond;
    }

    public function uninstall($sketchName,$target)
    {
        $request = new cf_sketch_request();
        $request->uninstall($sketchName, $target);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['uninstall'];
        return $respond;
    }

    public function defineParams($sketchName, $paramName, $params)
    {
        $request = new cf_sketch_request();
        $request->defineParam($sketchName, $paramName, $params);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['define'];
        return $respond;
    }


    public function undefineParams($paramNameArray=array())
    {
        $request = new cf_sketch_request();
        $request->undefineParam($paramNameArray);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['undefine'];
        return $respond;
    }



    public function defineEnvironment($envName, $environment)
    {
        $request = new cf_sketch_request();
        $request->defineEnvironment($envName, $environment);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['define_environment'];
        return $respond;
    }

    public function undefineEnvironment($environments=array())
    {
        $request = new cf_sketch_request();
        $request->undefineEnvironment($environments);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['undefine_environment'];
        return $respond;
    }

    public function getDefinitions()
    {
        $request = new cf_sketch_request();
        $request->getDefinitions();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['definitions'];
        return $respond;
    }

    public function getEnvironments()
    {
        $request = new cf_sketch_request();
        $request->getEnvironments();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['environments'];
        return $respond;
    }

    public function regenerate()
    {
        $request = new cf_sketch_request();
        $request->regenerate();
        return $this->run($request);
    }

    public function getValidations($key=null)
    {
        $request = new cf_sketch_request();
        $request->getValidations();
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['validations'];
        if (!empty($key))
        {
            $respond = !empty($respond[$key]) ? $respond[$key] : null;
        }
        return $respond;
    }
    public function validate($type, $data)
    {
        $request = new cf_sketch_request();
        $request->validate($type, $data);
        return $this->run($request);
    }

    public function deactivate($id,$target)
    {

        $activationDetails = $this->get_activation_details($id);

        $request = new cf_sketch_request();
        $request->deactivate($id);
        $apiResponse = $this->run($request);
        $apiArray = json_decode($apiResponse, true);
        $respond = $apiArray['api_ok']['data']['deactivate'];
        $this->_cleanupSketchDataAfterDeactivation($activationDetails,$target);
        return $respond;
    }

    /**
     * Remove parameters, environments and uninstall sketch if no activation is there.
     * @param  array $activationDetails
     * @return boolean
     */
    function _cleanupSketchDataAfterDeactivation($activationDetails,$target)
    {
         // remove parameters
        if (!empty($activationDetails['params']))
        {
            $this->undefineParams($activationDetails['params']);
        }

         // remove parameters
        if (!empty($activationDetails['environment']))
        {
           // check if environment is being used in other activations as well
           if (!$this->checkActivationEnvironment($activationDetails['identifier'],$activationDetails['environment']))
           {
                $env = array($activationDetails['environment']);
                $this->undefineEnvironment($env);
           }
        }

        $activations = $this->getActivations($activationDetails['sketchName']);
        if (empty($activations))
        {
            $this->uninstall($activationDetails['sketchName'],$target);
        }

        return true;
    }

  /**
   * get details of activation with id as parameter
   * @param  string $activationId unique identifier for the activation
   * @return array  array with activation details as params,environment else empty if not  found
   */
    function get_activation_details($activationId)
    {
        $allActivations = $this->getActivations();
        foreach ((array)$allActivations as $sketch=>$activations)
        {
            foreach ($activations as $activation)
            {
                if (!empty($activation['identifier']) && $activation['identifier'] == $activationId)
                {
                    // we have activation match, add sketch name as extra information
                    $activation['sketchName'] = $sketch;
                    return $activation;
                }
            }
        }
        return array();
    }

    /**
     * check if the environment is used in other activations as well
     * @param  string $activationId Activation id
     * @param  string $environment  Environment to check for
     * @return bool                 true if used else false
     */
    function checkActivationEnvironment($activationId,$environment)
    {
        $allActivations = $this->getActivations();
        foreach ((array)$allActivations as $sketch=>$activations)
        {
            foreach ($activations as $activation)
            {
                if (!empty($activation['environment']) && ($activation['environment'] == $environment) && $activation['identifier'] != $activationId)
                {
                   return true;
                }
            }
        }
        return false;
    }

    function initializeWorkSpace($username)
    {

        $workDir = get_dc_user_working_directory($username);
        $keyPath = get_dc_user_ssh_key_directory($username);
        log_message('debug',"Initializing worksapce with username $username and dir $workDir..");
        if (!is_dir($workDir))
        {
            log_message('debug',"creating workdir..");
            $result = @mkdir($workDir, 0700, true);
            if (!$result)
            {
                throw new Git_Settings_Exception("cannot create workdir on path: $workDir");
            }
        }

        if (!is_dir($keyPath))
        {
           log_message('debug',"creating keystore dir $keyPath");
           $keyResult =  @mkdir($keyPath, 0700, true);
           if (!$keyResult)
           {
            throw new Git_Settings_Exception("cannot create key store on path:  $keyPath");
           }
        }
        $this->generateApiConfigFile($username);
    }

    function generateApiConfigFile($username)
    {
        /**
         *  {
         * - log: "STDERR",
         * - log_level: 4,
         * - repolist: [ "../sketches" ],
         * - recognized_sources: [ "/var/cfengine/design-center/sketches" ],
         * - runfile: { location: "api-runfile.cf", standalone: false, relocate_path: "sketches" },
         * - vardata: "vardata.conf",
         * -}
         * */
        $repoListLocation = get_dc_user_repolist_location($username);
        $recognizedSources = get_dc_sketch_recognized_sources();
        $runfileLocation = get_dc_sketch_runfile_location($username);
        $vardataLocation = get_dc_sketch_vardatafile_location($username);
        $apiFilePath = get_dc_user_sketch_config_file_path($username);
        $constdata = get_dc_sketch_const_data();


        $configStructure = array(
            "log" => "STDERR",
            "log_level" => "3",
            "repolist" => array($repoListLocation),
            "recognized_sources" => $recognizedSources,
            "constdata"=> $constdata,
            "runfile" => array("location" => $runfileLocation, "standalone" => false, "relocate_path" => "sketches","filter_inputs" => array("cfengine_stdlib.cf")),
            "vardata" => $vardataLocation
        );
        $json = json_encode($configStructure);
        log_message('debug',"creating api config file at $apiFilePath with content $json");
        $result = file_put_contents($apiFilePath, $json);
    }

}

?>