<?php
class dc_rest_model extends Cf_Model
{
  const sketchResource = '/dc/sketch';
  const sketchActivationResource = '/dc/activation';
  const sketchDefinitionsResource = '/dc/definition';
  const sketchEnvironmentsResource = '/dc/environment';
  const sketchValidationResource = '/dc/validation';
  const sketchInstallResource = '/dc/install';
  const sketchValidateResource = '/dc/validate';
  const worspaceResource = '/dc/workspace';
  const worspaceCommitResource = '/dc/workspace/commit';
  const worspaceResetResource = '/dc/workspace/reset';
  const worspaceSettingsResource = '/dc/workspace/settings';
  /**
   * Gets all sketches
   *
   * @return array
   * @throws Exception
   */
  function getAllAvailableSketches()
  {

    try
    {
      $json = $this->getRestClient()->get(self::sketchResource);
      $returnArray = json_decode($json, true);

      return $returnArray['data'];
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  /**
   * [getSketch description]
   * @param  [type] $id [description]
   * @return [type]     [description]
   */
  function getSketch($id, $details = false)
  {

    try
    {
      $url = self::sketchResource . '/' . urlencode($id);
      $json = $this->getRestClient()->get($url);
      $returnArray = json_decode($json, true);

      return $returnArray['data'][0];
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  /**
   * Gets all sketches activations
   *
   * @return array
   * @throws Exception
   */
  function getActivations($sketchId = null, $details = true)
  {

    try
    {
      $url = self::sketchActivationResource;
      if ($sketchId)
      {
        $url = $url . '/?sketch=' . $sketchId.'&details=1';
      }
      $json = $this->getRestClient()->get($url);
      $returnArray = json_decode($json, true);

      return $returnArray['data'];
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  /**
   * Gets all sketches definitions
   *
   * @return array
   * @throws Exception
   */
  function getDefinitions()
  {

    try
    {
      $url = self::$sketchDefinitionsResource;
      $json = $this->getRestClient()->get($url);
      $returnArray = json_decode($json, true);

      return $returnArray;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  /**
   * Gets all sketches environment
   *
   * @return array
   * @throws Exception
   */
  function getEnvironments()
  {

    try
    {
      $url = self::sketchEnvironmentsResource;
      $json = $this->getRestClient()->get($url);
      $returnArray = json_decode($json, true);

      return $returnArray;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  /**
   * Gets all sketches validation
   *
   * @return array
   * @throws Exception
   */
  function getValidations($name = null,$filter=null)
  {

    try
    {
      $url = self::sketchValidationResource;
      if (!empty($name))
      {
        $url = $url . '/' . urlencode($name);
      }
      if(!empty($filter)) {
         $url = $url . '/?filter=' . urlencode($filter);
      }
      $json = $this->getRestClient()->get($url);
      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function install($sketchName)
  {

    try
    {
      $url = self::sketchResource;
      $url = $url . '/' . urlencode($sketchName);
      $json = $this->getRestClient()->put($url,array());
      $returnArray = json_decode($json, true);

      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function defineParams($sketchName, $paramName, $params)
  {
    $url = self::sketchDefinitionsResource;
    $url = $url . '/' . $paramName;
    $data['sketchName'] =  urlencode($sketchName);
    $data['params'] = $params;

    try
    {
      $json = $this->getRestClient()->put($url, $data);

      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function defineEnvironment($environmentName, $environment)
  {
    $url = self::sketchEnvironmentsResource.'/'.$environmentName;
    $data['environment'] = $environment;

    try
    {
      $json = $this->getRestClient()->put($url, $data);

      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function activate($sketchName, $envName, $bundleName, $paramName, $identifier, $metaData = array())
  {
    $id = urlencode($identifier);
    $url = self::sketchActivationResource . '/' .$id.'/'.$sketchName;
    $data['sketchName'] = $sketchName;
    $data['environmentName'] = $envName;
    $data['bundleName'] = $bundleName;
    $data['paramName'] = $paramName;
    $data['metaData'] = $metaData;

    try
    {
      $json = $this->getRestClient()->put($url, $data);

      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function deactivate($id, $sketchName)
  {
    $url = self::sketchActivationResource . '/' . urlencode($id).'/'.urlencode($sketchName);

    try
    {
      $json = $this->getRestClient()->delete($url);

      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function validate($validationType, $validationData)
  {
    $url = self::sketchValidateResource . '/' . urlencode($validationType);

    try
    {
      $json = $this->getRestClient()->post($url, $validationData);
      $response = json_decode($json, true);

      return $response;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function getWorkspace()
  {
    $url = self::worspaceResource;

    try
    {
      $json = $this->getRestClient()->get($url);
      $response = json_decode($json, true);

      return $response;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function commit($message, $userEmail)
  {
    $url = self::worspaceCommitResource;
    $data = array();
    $data['message'] = $message;
    $data['userEmail'] = $userEmail;

    try
    {
      $json = $this->getRestClient()->post($url, $data);
      $response = json_decode($json, true);

      return $response;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function reset_and_pull()
  {
    $url = self::worspaceResetResource;

    try
    {
      $json = $this->getRestClient()->post($url);
      $response = json_decode($json, true);

      return $response;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }

  function get_settings()
  {
    $url = self::worspaceSettingsResource;

    try
    {
      $json = $this->getRestClient()->get($url);
      $result = json_decode($json);
      return $result->data;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }

  function create_settings($data)
  {
    $url = self::worspaceSettingsResource;
    $postData = $data;

    try
    {
      $json = $this->getRestClient()->post_multipart($url,$postData);
      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }
  }
  function delete_settings()
  {

    $url = self::worspaceSettingsResource;

    try
    {
      $json = $this->getRestClient()->delete($url);
      return $json;
    }
    catch(Exception $e)
    {

      throw $e;
    }

  }
}
?>