<?php

namespace Services\Ldap\Implementations;


use Adldap\Connections\ConnectionInterface;
use Adldap\Connections\ProviderInterface;
use Services\Ldap\BaseLdap;

/**
 * Class LdapAuth
 * @package Services\Ldap\Implementations
 */
class Ldap extends BaseLdap
{

    /**
     * LdapAuth constructor.
     * @param array $config
     * @param ProviderInterface $provider
     * @param ConnectionInterface $connection
     */
    public function __construct(array $config, ProviderInterface $provider, ConnectionInterface $connection)
    {

        parent::__construct($config, $provider);
        $this->provider->setConfiguration($this->prepareConfig($config))->setConnection($connection);
    }

    /**
     * @param $username
     * @param $password
     * @return bool
     * @throws \Exception
     */
    public function bind($username, $password)
    {
        if ($this->provider->auth()->attempt($username, $password, true)) {
            return true;
        } else {
            throw new \Exception($this->provider->getConnection()->getLastError());
        }
    }

    public function getUserData($username)
    {
        return $this->provider->search()->rawFilter($this->config['ldap_filter'])->whereEquals($this->config['login_attribute'],
            $username)->firstOrFail();
    }

    public function getUsersList($searchArr = false)
    {
        try {
            $this->provider->auth()->bindAsAdministrator();
            $searchResult = $this->provider
                ->search()
                ->whereHas($this->config['login_attribute'])
                ->sortBy($this->config['login_attribute'], 'asc');

            if(!empty($this->config['ldap_filter'])){
                $searchResult =  $searchResult->rawFilter($this->config['ldap_filter']);
            }

            if ($searchArr) {
                $searchResult = $searchResult->whereContains($searchArr['searchKey'], $searchArr['searchValue']);
            }

            $searchResult = $searchResult->get();

            return $searchResult;
        } catch (\Adldap\Auth\BindException $e) {
            throw new \Exception($e->getMessage());
        }
    }


    public function getUser($name)
    {
        try {
            $this->provider->auth()->bindAsAdministrator();
            $searchResult = $this->provider
                ->search()
                ->where($this->config['login_attribute'] , '=', $name);

            if(!empty($this->config['ldap_filter'])){
                $searchResult =  $searchResult->rawFilter($this->config['ldap_filter']);
            }

            return $searchResult->firstOrFail();
        } catch (\Adldap\Auth\BindException $e) {
           throw new \Exception($e->getMessage());
        }
    }


    /**
     * @return array
     */
    private function prepareConfig()
    {
        $config = $this->config;
        if (isset($config)) {
            unset($config['login_attribute']);
            unset($config['ldap_filter']);
        }

        return $config;
    }

}