\set ON_ERROR_STOP true
-- DROP TABLE IF EXISTS ci_sessions;
DROP TABLE IF EXISTS ci_sessions;

CREATE TABLE "ci_sessions" (
        "id" varchar(128) NOT NULL,
        "ip_address" varchar(45) NOT NULL,
        "timestamp" bigint DEFAULT 0 NOT NULL,
        "data" text DEFAULT '' NOT NULL
);

CREATE INDEX "ci_sessions_timestamp" ON "ci_sessions" ("timestamp");

-- Table: users

-- DROP TABLE IF EXISTS users;

CREATE TABLE IF NOT EXISTS users
(
  id serial NOT NULL,
  username character varying(50),
  source character varying(20),
  last_login timestamp without time zone,
  remember_code character varying(50),
  dashboard integer DEFAULT NULL,
  CONSTRAINT id PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

ALTER TABLE users
ADD COLUMN IF NOT EXISTS seen_tour SMALLINT DEFAULT 0;

ALTER TABLE users
ADD COLUMN IF NOT EXISTS seen_wizard SMALLINT DEFAULT 0;

-- Table: astrolabeprofile

-- DROP TABLE IF EXISTS astrolabeprofile;

CREATE TABLE IF NOT EXISTS astrolabeprofile
(
  id serial NOT NULL,
  username character varying(50) NOT NULL,
  profileid character varying(50) NOT NULL,
  defaulttree boolean DEFAULT false,
  globaltree boolean DEFAULT false,
  sharedpermission character varying(50)[],
  sharedby character varying(50)[],
  data json,
  CONSTRAINT astrolabeprofile_pkey PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);


-- Table: custom_search

-- DROP TABLE IF EXISTS report CASCADE;

-- 3.6.2 introduce report_uniq constraint
ALTER TABLE IF EXISTS report
  DROP CONSTRAINT  IF EXISTS   report_uniq;

ALTER TABLE IF EXISTS report 
  ADD CONSTRAINT report_uniq UNIQUE (username, label);

CREATE TABLE IF NOT EXISTS report
(
  id serial NOT NULL,
  username character varying(50) NOT NULL,
  url character varying(500),
  reportType character varying(50),
  reportCategory character varying(50),
  type character varying(50) NOT NULL,
  readonly INTEGER NOT NULL default 0,
  is_public INTEGER NOT NULL default 0,
  can_subscribe INTEGER NOT NULL default 0,
  is_subscribed INTEGER NOT NULL default 0,
  label character varying(500) NOT NULL,
  date timestamp without time zone,
  params text NOT NULL,
  sharedPermission character varying(50)[],
  sharedby character varying(50)[],
  AdvancedReportsData json,
  CONSTRAINT report_pkey PRIMARY KEY (id),
  CONSTRAINT report_uniq UNIQUE (username, label)
)
WITH (
  OIDS=FALSE
);


-- DROP TABLE IF EXISTS report_schedule;


DO $$
BEGIN

IF EXISTS (SELECT 1
FROM information_schema.columns
WHERE  table_name='report_schedule' AND column_name='fromemail')
THEN
-- 3.6.1 renames fromemail TO emailfrom in report_schedule
ALTER TABLE IF EXISTS report_schedule RENAME COLUMN fromemail TO emailfrom;
END IF;

IF EXISTS (SELECT 1
FROM information_schema.columns
WHERE  table_name='report_schedule' AND column_name='toemail')
THEN
-- 3.6.1 renames fromemail TO emailfrom in report_schedule
ALTER TABLE IF EXISTS report_schedule RENAME COLUMN toemail TO emailfrom;
END IF;

IF EXISTS (SELECT 1
FROM information_schema.columns
WHERE  table_name='report_schedule' AND column_name='can_edit' AND column_name='can_delete' )
THEN
-- 3.6.1 removes can_edit and can_delete columns from report_schedule table
ALTER TABLE IF EXISTS report_schedule
  DROP COLUMN can_edit,
  DROP COLUMN can_delete;
END IF;
END$$;



CREATE TABLE IF NOT EXISTS report_schedule
(
  id character varying(500) NOT NULL,
  reportid int NOT NULL references report(id) ON DELETE CASCADE,
  userid character varying(50) NOT NULL,
  title character varying(500) NOT NULL,
  description character varying(500) NOT NULL,
  emailfrom character varying(500) NOT NULL,
  emailto character varying(500) NOT NULL,
  enabled INTEGER NOT NULL default 1,
  query text NOT NULL,
  outputtypes character varying(50)[],
  schedule character varying(500) NOT NULL,
  schedulehumanreadabletime character varying(500) NOT NULL,
  schedulename character varying(500) NOT NULL,
  site_url text NOT NULL,
  hostcontextsprofileid character varying(20),
  hostcontextspath text default NULL,
  hostcontexts json DEFAULT NULL,
  scheduledata json,
  CONSTRAINT report_schedule_pkey PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

ALTER TABLE report_schedule
ADD COLUMN IF NOT EXISTS excludedhosts json DEFAULT NULL;

ALTER TABLE "report_schedule"
ADD COLUMN IF NOT EXISTS skipmailing boolean NULL DEFAULT 'false';

-- DROP TABLE IF EXISTS app;

CREATE TABLE IF NOT EXISTS app
(
  displayindex integer,
  filepath text,
  hascontroller integer,
  icon character varying(50),
  meta json,
  showappslist integer,
  state integer,
  url text,
  id character varying(100),
  CONSTRAINT app_pk PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);


-- Table: eventslog

-- DROP TABLE IF EXISTS eventslog;

CREATE TABLE IF NOT EXISTS eventslog
(
  id serial NOT NULL,
  username character varying(100),
  item_id character varying(100), -- Alert id
  item_type character varying,
  item_name character varying(500),
  tags character varying(500)[],
  "time" timestamp without time zone,
  severity character varying(20),
  message text,
  status   character varying(10),
  CONSTRAINT eventslog_pkey PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

-- 3.7.0 dashboard sharing
-- DROP TABLE IF EXISTS dashboard_dashboards;
CREATE TABLE IF NOT EXISTS dashboard_dashboards
(
  id serial NOT NULL,
  name character varying(200),
  username character varying(20),
  public integer DEFAULT 0,
  widgets character varying(200),
  CONSTRAINT dashboard_dashboards_pkey PRIMARY KEY (id)
)
WITH (
  OIDS=FALSE
);

ALTER TABLE IF EXISTS  dashboard_dashboards
ADD COLUMN IF NOT EXISTS  sharedWith jsonb NULL
DEFAULT '{"roles": [], "users": [], "sharedWithAll": true}';

-- DROP TABLE IF EXISTS dashboard_widgets CASCADE;

-- 3.6.1 intruduced ordering column to dashboard_widgets
ALTER TABLE IF EXISTS dashboard_widgets
  ADD COLUMN IF NOT EXISTS ordering integer DEFAULT 0;


CREATE TABLE IF NOT EXISTS dashboard_widgets
(
  id serial NOT NULL,
  name character varying(500),
  type character varying(20),
  username character varying(50),
  ordering integer DEFAULT 0,
  dashboardid integer,
  CONSTRAINT dashboard_widgets_pkey PRIMARY KEY (id),
  CONSTRAINT dashboard_widgets_dashboardid_fkey FOREIGN KEY (dashboardid)
      REFERENCES dashboard_dashboards (id)  ON DELETE CASCADE
)
WITH (
  OIDS=FALSE
);

ALTER TABLE IF EXISTS "dashboard_widgets"
ADD COLUMN IF NOT EXISTS payload json NULL DEFAULT '{}';

CREATE INDEX IF NOT EXISTS _ordering ON dashboard_widgets (ordering) WITH (FILLFACTOR = 90);

-- DROP TABLE IF EXISTS dashboard_rules CASCADE;

CREATE TABLE IF NOT EXISTS dashboard_rules
(
  id serial NOT NULL,
  name character varying(200),
  description character varying(1000),
  type character varying(20),
  username character varying(20),
  policyconditions json,
  sketchactivation json,
  inventoryconditions json,
  softwareupdateconditions json,
  CONSTRAINT dashboard_rules_pkey PRIMARY KEY (id),
  CONSTRAINT dashboard_rules_name_key UNIQUE (name,username)
)
WITH (
  OIDS=FALSE
);

-- 3.12.0 add new alert custom condition
ALTER TABLE IF EXISTS dashboard_rules
  ADD COLUMN IF NOT EXISTS customconditions json DEFAULT '{}';

-- DROP TABLE IF EXISTS dashboard_alerts;

CREATE TABLE IF NOT EXISTS dashboard_alerts
(
  id serial NOT NULL,
  ruleid integer,
  failhosts integer DEFAULT 0,
  lastcheck integer,
  lasteventtime integer,
  laststatuschange integer,
  servertime integer,
  pause integer DEFAULT 0,
  paused integer DEFAULT 0,
  name character varying(500),
  severity character varying(10),
  site_url text,
  status character varying(32),
  totalhosts integer,
  username character varying(50),
  widgetname character varying(100),
  emailtonotify character varying(100),
  reminder integer,
  widgetid integer,
  hostcontextsprofileid character varying(20),
  hostContexts json DEFAULT NULL,
  hostContextsPath text default NULL,
  CONSTRAINT dashboard_alerts_pkey PRIMARY KEY (id),
  CONSTRAINT dashboard_alerts_ruleid_fkey FOREIGN KEY (ruleid)
      REFERENCES dashboard_rules (id)  ON DELETE CASCADE,
  CONSTRAINT dashboard_alerts_widgetid_fkey FOREIGN KEY (widgetid)
      REFERENCES dashboard_widgets (id)  ON DELETE CASCADE,
  CONSTRAINT dashboard_alerts_uniq UNIQUE (username, name, widgetid)
)
WITH (
  OIDS=FALSE
);

ALTER TABLE dashboard_alerts
ADD COLUMN IF NOT EXISTS excludedhosts json DEFAULT NULL;

-- DROP TABLE IF EXISTS variables_dictionary;

-- 3.6.2 introduce variables_dictionary_attribute_name constraint
ALTER TABLE IF EXISTS variables_dictionary
  DROP CONSTRAINT  IF EXISTS   variables_dictionary_attribute_name;

ALTER TABLE IF EXISTS variables_dictionary 
  ADD CONSTRAINT variables_dictionary_attribute_name UNIQUE (attribute_name);

CREATE TABLE IF NOT EXISTS variables_dictionary
(
  id serial NOT NULL,
  attribute_name character varying(200),
  category character varying(200),
  readonly integer default 0,
  type character varying(200),
  convert_function character varying(200) NULL,
  CONSTRAINT variables_dictionary_pkey PRIMARY KEY (id),
  CONSTRAINT variables_dictionary_attribute_name UNIQUE (attribute_name)
)
WITH (
  OIDS=FALSE
);

-- Table: dashboard_scripts

CREATE TABLE IF NOT EXISTS dashboard_scripts
(
  id serial NOT NULL,
  name text,
  description text,
  script_name text,
  type text,
  CONSTRAINT unique_script_name UNIQUE (name),
  CONSTRAINT dashboard_scripts_pkey PRIMARY KEY (id)
)
WITH (
OIDS=FALSE
);


DROP TABLE IF EXISTS dashboard_alerts_script;

CREATE TABLE IF NOT EXISTS dashboard_alerts_script
(
  alert_id integer NOT NULL,
  script_id integer,
  CONSTRAINT alert_id_fk FOREIGN KEY (alert_id)
  REFERENCES dashboard_alerts (id) MATCH SIMPLE
  ON UPDATE NO ACTION ON DELETE CASCADE,
  CONSTRAINT script_id_fk FOREIGN KEY (script_id)
  REFERENCES dashboard_scripts (id) MATCH SIMPLE
  ON UPDATE NO ACTION ON DELETE CASCADE
)
WITH (
OIDS=FALSE
);


-- Index: fki_alert_id_fk
CREATE INDEX IF NOT EXISTS fki_alert_id_fk
ON dashboard_alerts_script
USING btree
(alert_id);

-- Index: fki_script_id_fk
CREATE INDEX IF NOT EXISTS fki_script_id_fk
ON dashboard_alerts_script
USING btree
(script_id);

CREATE TABLE IF NOT EXISTS mail_settings (
    key character varying PRIMARY KEY,
    value text
);

CREATE TABLE  IF NOT EXISTS customization (
   key character varying PRIMARY KEY,
   value text
);
COMMENT ON TABLE customization IS 'Stores Mission Portal UI customization config.';

GRANT ALL PRIVILEGES ON DATABASE cfmp TO cfmppostgres;
GRANT ALL PRIVILEGES ON ALL TABLES IN SCHEMA public TO cfmppostgres;
GRANT ALL PRIVILEGES ON ALL SEQUENCES IN SCHEMA public TO cfmppostgres;
