'use strict';
/*
 * Functions which produce SQL string from SQL scope
 *
 */

reports.factory('sqlHelper', function($rootScope) {

    var SQL = {};

    var $scope = {};

    $scope.Comparison  = Comparison;
    $scope.Logical     = Logical;
    $scope.Order       = Order;
    $scope.Limit       = Limit;


    SQL.setSQLScope = function (SQLScope) {
        $scope.SQL = SQLScope;
    };

    /* ---------------------SQL FUNCTIONS  ---------------------------------- */

    SQL.getSQL_SELECT  = function() {

        var select = 'SELECT ';
        var tmp    = new Array();

        // check if object is empty with _underscore.js
        if(_.isEmpty($scope.SQL.fields)) {
            return false;
        }

        // join fields into tmp array
        angular.forEach($scope.SQL.fields, function(field, key) {
            var fieldWithAlias = '';

            switch (field.dataType) {
                case 'real':
                case 'integer':
                    fieldWithAlias = 'cast(' + field.sqlField + ' AS ' + field.dataType + ') AS "' + field.label + '"';
                    break;
                default:
                    fieldWithAlias = field.sqlField + ' AS "' + field.label + '"';
            }

            tmp.push(fieldWithAlias);
        });

        if(tmp.length > 0) {
            //select = select + '\'' + tmp.join('\',\'') + '\'';  // wrap with quotes
            select = select +  tmp.join(', ');  // wrap with quotes
            return select;
        }

        return false;
    };

    SQL.getSQL_FROM    = function() {
        var from = 'FROM ';
        var tableList  = new Array();
        var tablesByPimaryKey = {};
        var tablesByForeignKey = {};
        var foreignJoin = '';

        var hackTableName = '';

        var joins = [];
        var joinsIndex =  0;


        if (!$scope.SQL.fields) {
            return false;
        }

        // CHECK FILTER

        //*********************************************************************//
        var $fields_from_filter = {};
        var selectedComparison = {};
        if ($scope.SQL.filters)
        {
            angular.forEach($scope.SQL.filters, function(field, key)
            {
                // no comparision selected
                if (field.comparison === undefined)
                {
                    return; // works as continue
                }
                //trying to find if this comparision exist
                selectedComparison = _.find($scope.Comparison, function (item) {
                    return item.value === field.comparison;
                });

                // check if this comparison required value (field > value), for example  a=5, a<6, if ignoreempty=false - this mean that value is required
                if (selectedComparison.ignoreempty === true && (field.value === '' || field.value === undefined) ) {
                    return;  // works as continue
                }

                $fields_from_filter[field.table] = {};
                $fields_from_filter[field.table] = field;

            }); // end foreach
        }
        //*********************************************************************//

        var totalTables = {};

        angular.forEach($scope.SQL.fields, function(field, key){
            totalTables[field.table] = field;
        });

        angular.forEach($fields_from_filter, function(field, key){
            totalTables[field.table] = field;
        });


        // join fields into tmp array
        angular.forEach(totalTables, function(field, key){
            if ($.inArray(field.table, tableList) === -1) {
                tableList.push(field.table);
            }
        });

        var length = tableList.length;
        for(var i=length-1; i >= 0; i--) {

            if (DCA[tableList[i]] === undefined) {
                alert('Table ' + tableList[i] + ' not found. Please check your DCA_TABLE array');
                tableList.splice(i, 1); // remove this field
                continue;
            }

            // check if this table has pk, and if so, try to find another tables with the same pk
            if (DCA[tableList[i]]['Keys'].primary_key !== undefined && DCA[tableList[i]]['Keys'].primary_key !== '' && DCA[tableList[i]]['Keys'].primary_key !== null) {

                // try to find another tables and join them
                var pk_key = DCA[tableList[i]]['Keys'].primary_key;

                if(tablesByPimaryKey[pk_key] === undefined) {
                    tablesByPimaryKey[pk_key]  = [];
                }

                tablesByPimaryKey[pk_key].push(tableList[i]);


                //delete table from list
                tableList.splice(i, 1);
            }
        }

        // flag to indicae if we deleted PromiseStatusLast
        var deleted  = false;

        //special case for foreign_keys
        if(totalTables.PromiseStatusLast !== undefined && totalTables.PromiseDefinitions !== undefined) {
            foreignJoin = '  PromiseDefinitions ON PromiseDefinitions.PromiseHandle = PromiseStatusLast.PromiseHandle';

            //delete PromiseHandle key - ref to PromiseStatusLast table
            tablesByPimaryKey['HostKey'].splice($.inArray('PromiseStatusLast', tablesByPimaryKey['HostKey']),1)
            deleted = true;
        }

        //now we have 2 arrays - tablesByPimaryKey contain all tables which can be joined by primary key,
        //and tableList with tables without PK, or tables without pair for join

        angular.forEach(tablesByPimaryKey, function(tables, pk_key)
        {
            var length   = tables.length;
            for(var i=0;  i < length; i++)
            {
                if (tables[i]  !== 'PromiseStatusLast' && tables[i] !== 'PromiseDefinitions')
                {
                    hackTableName = tables[i];
                    break;
                }
            }
        });

        angular.forEach(tablesByPimaryKey, function(tables, pk_key){

            var length   = tables.length;
            var join_str = '';


            if (length >=2)
            { // we can join this tables
                for(var i=0;  i < length; i++) {
                    if (i === 0)
                    {
                        join_str = tables[i];
                    }
                    else
                    {
                        join_str +=  ' INNER JOIN ' + tables[i] + ' ON ' + tables[i] + '.' + pk_key  + ' = ' + tables[i-1] + '.' +  pk_key;
                    }
                }
                joins.push(join_str);
            }
            else { // return this table to the tableList
                if (foreignJoin === '' || (deleted === true && tables[0] !== 'PromiseDefinitions')) {
                    tableList.push(tables[0]);
                    deleted = false;
                }
            }

        });

        var superarray = joins.concat(tableList);
        var from_tmp = '';

        from_tmp += superarray.join(', ');


        if (from_tmp.length>0 && foreignJoin.length > 0)
        {
            if (hackTableName !== '')
            {
                from_tmp += ' INNER JOIN PromiseStatusLast ON PromiseStatusLast.HostKey = ' + hackTableName + '.HostKey' + ' INNER JOIN ' + foreignJoin;
            }
        }
        else if (from_tmp.length===0 && foreignJoin.length > 0)
        {
            from_tmp = ' PromiseDefinitions INNER JOIN PromiseStatusLast ON PromiseDefinitions.PromiseHandle = PromiseStatusLast.PromiseHandle';
        }


        from += from_tmp;


        return from;
    };

    SQL.getSQL_WHERE   = function() {
        var where = 'WHERE ', fieldName = '';
        var tmp   = new Array();


        var selectedComparison = false;
        var selectedLogical    = false;


        // check if object is empty with _underscore.js
        if(_.isEmpty($scope.SQL.filters)) {
            return false;
        }

        angular.forEach($scope.SQL.filters, function(field, key){

            // no comparision selected
            if (field.comparison === undefined)
            {
                return; // works as continue
            }

            selectedComparison = _.find($scope.Comparison, function (item) {
                return item.value === field.comparison;
            });

            // check if this coparison required value (field > value), for example  a=5, a<6, if ignoreempty=false - this mean that value is required
            if (selectedComparison.ignoreempty === true && (field.value === '' || field.value === undefined) ) {
                return;  // works as continue
            }

            var str = '';
            var arr = [];



            // TODO:
            // find real sanitize method
            // clonedFrom has original field name
            fieldName = (field.clonedFrom !== undefined ? field.clonedFrom : field.sqlField);

            if (selectedComparison.ignoreempty === true)  // for comparision which require argument
            {
                // wrap value with quotes
                var  quotes = '';
                if(field.comparison === 'ILIKE' || field.comparison === 'NOT ILIKE') {
                    quotes = "'";
                }
                else if (field.dataType !== 'integer' && field.dataType !== 'real') {
                    quotes = "'";
                }

                switch (field.comparison) {
                    case "IN":
                    case "NOT IN":
                        str  = fieldName + ' ' + field.comparison + ' ' + '('+ (field.value) + ')' ;
                        break;

                    case "ANY":
                    case "ALL":
                        str  = quotes + field.value + quotes  + "=" + field.comparison + "(" + fieldName + ')' ;
                        break;

                    default:
                        str  = fieldName + ' ' + field.comparison + ' ' +  quotes + (field.value) + quotes ;
                }


            }
            else { // for comparision which not require argument, like is null
                str  += fieldName + ' ' + field.comparison;
            }

            arr.condition = str;
            arr.logical   = field.logical;

            tmp.push(arr);

        }); // end foreach


        // return where strin, cut logical OR | AND from last filter
        if(tmp.length >0) {
            var length = tmp.length;
            var i = 0;
            for (i=0; i<length; i++)
            {
                if (i< (length-1) )
                {
                    where += ' ' + tmp[i].condition + ' ' + tmp[i].logical;
                }
                else
                {
                    where += ' ' + tmp[i].condition;
                }
            }
            return where;
        }


        return false;
    };

    SQL.getSQL_GROUPBY = function() {

        var groupby = 'GROUP BY ';
        var tmp    = new Array();

        // check if object is empty with _underscore.js
        if(_.isEmpty($scope.SQL.groupby)) {
            return false;
        }

        // join fields into tmp array
        angular.forEach($scope.SQL.groupby, function(field, key){
            // for real fields, we use their sqlfield name
            // for cloned fields, we use their name (alias)
            if(field.clonedFrom === undefined)
            {
                tmp.push(field.sqlField);
            }
            else
            {
                tmp.push(field.name);
            }
        });

        if(tmp.length > 0) {
            //groupby = groupby + '\'' + tmp.join('\',\'') + '\'';  // wrap with quotes
            groupby = groupby +  tmp.join(', ');  // wrap with quotes
            return groupby;
        }

        return false;
    };

    SQL.getSQL_ORDERBY = function() {

        var orderby = 'ORDER BY ';
        var tmp    = new Array();


        // check if object is empty with _underscore.js
        if(_.isEmpty($scope.SQL.orderby)) {
            return false;
        }

        // join fields into tmp array
        angular.forEach($scope.SQL.orderby, function(field, key){
            // for real fields, we use their sqlfield name
            // for cloned fields, we use their name (alias)

            var str = ' ' + field.order; // get order direction

            if(field.clonedFrom === undefined)
            {
                tmp.push(field.sqlField + str);
            }
            else
            {
                tmp.push(field.name + str);
            }
        });

        if(tmp.length > 0) {
            orderby = orderby + tmp.join(', ');  // wrap with quotes
            return orderby;
        }

        return false;
    };

    SQL.getSQL_LIMIT   = function() {
        var limit = '';
        if ($scope.SQL.limit !== null && $scope.SQL.limit !== '' && $scope.SQL.limit !== undefined) {
            limit = 'LIMIT ' + $scope.SQL.limit;
        }
        return limit;
    };

    SQL.getSQL_FULL    = function() {
        var select  = '';
        var from    = '';
        var where   = '';
        var groupby = '';
        var orderby = '';
        var limit   = '';

        select = SQL.getSQL_SELECT();
        if (select !== false) {
            $scope.SQL.SQLSTRING  = select;
        }
        else {
            $scope.SQL.SQLSTRING = [];
            return false; // we can't have valid SQL without select
        }


        from = SQL.getSQL_FROM();
        if (from !== false)
        {
            $scope.SQL.SQLSTRING += ' '+from;
        }

        where = SQL.getSQL_WHERE();
        if(where !== false) {
            $scope.SQL.SQLSTRING += ' ' + where;
        }

        groupby = SQL.getSQL_GROUPBY();
        if(groupby !== false) {
            $scope.SQL.SQLSTRING += ' ' + groupby;
        }

        orderby = SQL.getSQL_ORDERBY();
        if(orderby !== false) {
            $scope.SQL.SQLSTRING += ' ' + orderby;
        }


        limit = SQL.getSQL_LIMIT();
        if(limit !== false) {
            $scope.SQL.SQLSTRING += ' ' + limit;
        }
        if($scope.SQL.SQLSTRING !=='' && $scope.SQL.SQLSTRING !== false)
        return $scope.SQL.SQLSTRING.trim();
        else
        return '';
    };

    return SQL;
});