'use strict';
/*
 * Service to work with http
 *
 **/
var reportServiceClass = Class.extend({
    ADV_REPORTS_URL:  MP_SITE_URL + '/advancedreports',
    loadReports: function(pagingData) {
        return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/listsearches',
            data: pagingData,
            cache: false
        });
    },
    loadAllReports: function(pagingData) {
        return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/listallsearches',
            data: pagingData,
            cache: false
        });
    },
    loadReport: function(id) {
        var report = this.ADV_REPORTS_URL + '/load/' + encodeURIComponent(id);
        return this.$http.get(report, {
            cache: false
        });
    },
    loadInventoryAlert: function(id) {
        var report = this.ADV_REPORTS_URL + '/loadalert/' + encodeURIComponent(id);
        return this.$http.get(report, {
            cache: false
        });
    },
    runQuery: function(data) {
       return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL +'/results/',
            data: data,
            cache: false
        });
    },
    saveReport: function(data) {
        return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/save',
            data: data,
            cache: false
        });
    },
    deleteReport: function(id) {
        var result = this.ADV_REPORTS_URL + '/delete/' + encodeURIComponent(id);
        return this.$http.get(result, {
            cache: false
        });
    },
    saveScheduleForReport: function(data) {
        return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/savescheduleforreport',
            data: data,
            cache: false
        });
    },
    loadSchedules: function(id) {
          return this.$http.get(this.ADV_REPORTS_URL + '/loadreportschedules/' + encodeURIComponent(id), {
            cache: false
        });
    },
    //data = reportid, scheduleid
    deleteSchedule: function(data) {
         return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/deleteschedule',
            data: data,
            cache: false
        });
    },
    subscribeToReport: function(data) {
         return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/subscribe',
            data: data,
            cache: false
        });
    },
    unsubscribeFromReport: function(data) {
         return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/unsubscribe',
            data: data,
            cache: false
        });
    },
    copyReport: function(data) {
         return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/copyReport',
            data: data,
            cache: false
        });
    },

    deleteHosts: function(listOfKeys)
    {
        var keys = listOfKeys || [];
        var data = $.param({'delhosts':keys});
        if (keys.length)
        {
          return this.$http({
            method: 'POST',
            url: MP_SITE_URL + '/host/delhosts',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            data: data,
            cache: false
        });
        }
        return false;
    },

    getInventoryFields: function(data) {
        var url = this.ADV_REPORTS_URL + '/getinventoryfields';
        return this.$http.get(url, {cache: false});
    },
    updateVariablesDictionary: function(fields) {
        var data = $.param({
            fields  : fields
        });
         return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/updatevariablesdictionary',
            data: data,
            cache: false
        });
    },

    runChangesQuery: function(queryData) {
        var data = $.param(queryData);
        return this.$http({
            method: 'POST',
            headers : {
                'Content-Type':'application/x-www-form-urlencoded; charset=UTF-8'
            },
            url: this.ADV_REPORTS_URL + '/changes',
            data: data,
            cache: false
        });
    }
});



(function() {
    var reportServiceProvider = Class.extend({
        instance: new reportServiceClass(),
        $get: ['$http', function($http) {
            this.instance.$http = $http;
            return this.instance;
        }]
    });




    angular.module('reportServiceModule', [])
        .provider('reportService', reportServiceProvider);


}());