/**
 * environment Directive
 *
 * Use of Class.js
 *
 */
var environmentSelectionDirective = BaseController.extend({
    /**
     * @override
     */

    selectClassOne: null,
    operator: null,
    selectClassTwo: null,
    init: function($scope, $elm, $attrs, naviTreeService, $filter, $modal) {
        var self = this;
        this._element = $elm;
        this.filter = $filter;
        this._attrs = $attrs;
        this._naviTreeService = naviTreeService;
        this.$modal = $modal;
        this._super($scope);
        this.$scope.totalHostList = [];

        this.$scope.treeProfiles = [];

        this._naviTreeService.getTreeProfiles().then(

        function(result) {
            self.$scope.treeProfiles = result.data;
        },

        function(error) {
            self.$scope.treeProfiles = [];
        });
    },
    /**
     * @override
     */
    defineListeners: function() {
        var self = this;
        var $scope = self.$scope;

        $scope.$watch('env', function(val) {
           if (!val.name) self.addAllHostClass();
        },true);

        // watch the context changes in free context form
        $scope.$watch('context', function(val) {
            if (!val) {
                return;
            }

            self.$scope.nodeLevels = []; // reset nodes
            self.$scope.selectedNodeLevel = [];
            self.$scope.selectedProfile = null;

            var nodes = [];

            angular.forEach($scope.context, function(value, key) {
                nodes.push({
                    'label': value,
                    'classRegex': value
                });
            });

            self._updateClassEvaluation(nodes);
        });


    },
    defineScope: function() {
        var self = this;
        this.$scope.selectedProfile = {};
        this.$scope.treeNodes = [];
        this.$scope.nodeLevels = [];
        this.$scope.selectedNodeLevel = [];
        this.$scope.selectedNode = {};
        this.$scope.hostFilterSet = false;
        this.$scope.showHosts = false;
        this.$scope.showHostButton = false;


        this.$scope.updateTreeProfile = this.updateTreeProfile.bind(this);
        this.$scope.addNodeLevel = this.addNodeLevel.bind(this);
        this.$scope.showHost = this.showHost.bind(this);
        this.$scope.addAllHostClass = this.addAllHostClass.bind(this);
        this.$scope.clearContextFields = this.clearContextFields.bind(this);

    },

    clearContextFields: function() {
        this.$scope.nodeLevels = []; // reset nodes
        this.$scope.selectedNodeLevel = [];
        this.$scope.context = null;
        this.$scope.env.name = null;
        this.$scope.env.includes = null;
        this.$scope.selectedProfile = null;

    },

    updateTreeProfile: function(tree) {
        var self = this;
        this.$scope.selectedProfile = angular.copy(tree);
        this.$scope.showHosts = false;

        this._naviTreeService.getTreeNodes(tree).then(

        function(result) {
            self.$scope.treeNodes = result.data;
            self.$scope.nodeLevels = []; // reset nodes
            self.$scope.selectedNodeLevel = [];
            self.$scope.nodeLevels.push(result.data);
            self.$scope.showHostButton = false;
        },

        function(error) {
            self.$scope.treeNodes = [];
        });


    },

    addNodeLevel: function(node, index) {
        this.$scope.showHosts = false;

        // remove everthing after the index;
        var startIndex = (index == 0) ? 0 : index;


        this.$scope.nodeLevels = this.$scope.nodeLevels.slice(0, startIndex + 1);
        this.$scope.selectedNodeLevel = this.$scope.selectedNodeLevel.slice(0, startIndex + 1);
        this.$scope.selectedNodeLevel[index] = node;
        this.$scope.selectedNode = node;
        this.$scope.context = null;

        this._updateClassEvaluation(this.$scope.selectedNodeLevel);
        if (node.children && node.children.length > 0) {
            this.$scope.nodeLevels.push(node.children);
        }
    },

    showHost: function() {
        var self = this;
        this.hostListModalInstance = this.$modal.open({
            templateUrl: 'hostSelectionModal.html',
            backdrop: 'static',
            keyboard: true,
            controller: function($scope, $modalInstance, naviTreeService, $filter, env, dependOs) {
                var self = this;
                $scope.totalHostList =   [];
                $scope.originalHostList = [];
                $scope.selectedHosts = {};
                $scope.hostSearch = {};
                $scope.hostSearch.input = '';
                $scope.allchecked = true;
                $scope.showHosts = false;

                $scope.checkAllHosts = function() {

                    if ($scope.allchecked) {
                        $scope.selectedHosts = {};
                        $('.host-selector').attr('checked',false);
                    } else {
                        $scope.setAllSelected();
                        $('.host-selector').attr('checked',true);
                    }
                    $scope.allchecked = !$scope.allchecked;
                };

                $scope.disableButton = function() {
                    //checks if selected host list is empty
                    var empty = angular.equals({},$scope.selectedHosts);
                    return empty;
                };

                $scope.runHostSearch = function (event) {
                    $scope._filterHosts();
                };

                $scope._filterHosts = function (event) {
                    $scope.searchUsedFlag = true;

                    var filteredHosts = [];
                    angular.forEach($scope.originalHostList, function (field, key) {
                        if (field[1].indexOf($scope.hostSearch.input) !== -1) {
                           filteredHosts.push(field);
                        }
                    });


                    if ($scope.hostSearch.input !== undefined && $scope.hostSearch.input.length > 0) {
                        $scope.totalHostList = filteredHosts;
                        $scope.selectedHosts = {};
                        $scope.setAllSelected();
                        $scope.disableButton();
                    } else {
                        $scope.totalHostList = $scope.originalHostList;
                    }
                };


                $scope.showOnlyUnSupportedHost = function() {
                    $scope.hostFilterSet = true;
                    $scope.totalHostList = $filter('filter')($scope.totalHostList, {
                        '2': '0'
                    });
                };

                $scope.close = function() {
                    $modalInstance.dismiss('cancel');
                };

                $scope.hostMarkAsSelected = function(host) {
                    var name = host[1];
                    var key = host[0];
                    var classKey = key.replace("SHA=", "PK_SHA_");
                    if ($scope.selectedHosts[classKey]) {
                        $scope.allchecked = false;
                        delete $scope.selectedHosts[classKey];
                    } else {
                        $scope.selectedHosts[classKey] = name;
                    }
                    $scope.disableButton();

                };

                $scope.useSelectedHost = function() {

                    $modalInstance.close($scope.selectedHosts);
                };

                $scope.resetUnSupportedHostFilter = function() {
                    $scope.hostFilterSet = false;
                    $scope.totalHostList = $scope.originalHostList;
                };

                $scope.setAllSelected = function() {
                    //populate selectedHost list so all hosts in group are checked by default
                    angular.forEach($scope.totalHostList, function(value,key){
                        $scope.hostMarkAsSelected(value);
                    });
                };

                var getNodeHostSuccess = function(result) {
                        $scope.totalHostList = result.data.rows;
                        $scope.originalHostList = angular.copy($scope.totalHostList);
                        $scope.setAllSelected();
                        $scope.showHosts = true;
                    };

                var getNodeHostfailure = function(error) {
                        $scope.hostListError = error.data;
                        $scope.totalHostList = [];
                        $scope.selectedHosts = {};
                        $scope.showHosts = true;
                    };


                naviTreeService.getNodeHosts(env, dependOs).then(getNodeHostSuccess, getNodeHostfailure);


            },
            resolve: {
                naviTreeService: function() {
                    return self._naviTreeService;
                },
                $filter: function() {
                    return self.filter;
                },
                env: function() {
                    return self.$scope.env;
                },
                dependOs: function() {
                    return self.$scope.sketch.metadata.depends.os;
                }
            }
        });


        this.hostListModalInstance.result.then(function(selectedHosts) {
            var nodes = [];
            var regex = Object.keys(selectedHosts).join('|');
            var names = [];
            for (var key in selectedHosts) {
                var val = selectedHosts[key];
                names.push(val);
            }
            var envName = names.join(', ');
            nodes.push({
                'classRegex': regex,
                'label':envName
            });


            self._updateClassEvaluation(nodes);
        });


    },

    addAllHostClass: function() {
        this.clearContextFields();
        this.$scope.env.name = 'cfengine_3';
        this.$scope.env.includes = ['cfengine_3'];
        this.$scope.selectedProfile = {
            name: 'All hosts'
        };
        this.$scope.showHostButton = true;
    },

    _updateClassEvaluation: function(nodes) {
        var environmentLabel =  '';
        var regex = [];
        var label = [];
        angular.forEach(nodes, function(val) {
            regex.push(val.classRegex);
            label.push(val.label);
        });

        environmentLabel =  label.join(' and ');


        this.$scope.env.name = environmentLabel;
        this.$scope.env.includes = regex;
    }


});