var sketchService = Class.extend({
    LOAD_SKETCH_CATEGORY_URL: MP_SITE_URL + '/designcenter/dcapi/sketchCategory/',
    LOAD_SKETCH_URL: MP_SITE_URL + '/designcenter/dcapi/sketch/',
    LOAD_CLASSES_URL: MP_SITE_URL + '/designcenter/dcapi/classes/',
    ACTIVATE_SKETCH_URL: MP_SITE_URL + '/designcenter/dcapi/activate/',
    DEACTIVATE_SKETCH_URL: MP_SITE_URL + '/designcenter/dcapi/deactivate/',

    ACTIVATION_DETAILS_SKETCH_URL: MP_SITE_URL + '/designcenter/dcapi/activation_details/',
    COMMIT_CHANGES_URL: MP_SITE_URL + '/designcenter/dcapi/commit/',
    SKETCH_VALIDATE_URL: MP_SITE_URL + '/designcenter/dcapi/validate/',
    SKETCH_VALIDATION_URL: MP_SITE_URL + '/designcenter/dcapi/validation/',
    GIT_RESET_URL: MP_SITE_URL + '/designcenter/dcapi/git_reset/',
    PROMISE_NOT_KEPT_URL: MP_SITE_URL + '/designcenter/dcapi/promise_notkept_log/',

    /**
     * Query server and returns us sketch
     * information.
     * @return Service handler
     */
    loadSketch: function(id) {
        var loadSketchUrl = this.LOAD_SKETCH_URL + id;
        return this.$http.get(loadSketchUrl, {
            cache: false
        });
    },
    /**
     * Query server and returns us sketch
     * information.
     * @return Service handler
     */
    loadSketchCategories: function() {
        return this.$http.get(this.LOAD_SKETCH_CATEGORY_URL, {
            cache: false
        });
    },
    getClasses: function() {
        return this.$http.get(this.LOAD_CLASSES_URL, {
            cache: false
        });
    },
    activateSketch: function(sketchId, params, environment) {
        var data = {
            'sketchId': sketchId,
            'params': params,
            'environment': environment
        };
        return this.$http.post(this.ACTIVATE_SKETCH_URL, data, {});
    },
    deactivateSketch: function(identifier, sketchName) {
        var url = this.DEACTIVATE_SKETCH_URL + encodeURIComponent(identifier) + '/' + encodeURIComponent(sketchName) ;
        var data = {};
        return this.$http.post(url, data, {});
    },

    commitChanges: function(msg) {
        var data = {
            'commitMessage': msg
        };
        return this.$http.post(this.COMMIT_CHANGES_URL, data, {});
    },
    getActivationDetails: function(sketchName) {
        var url = this.ACTIVATION_DETAILS_SKETCH_URL + encodeURIComponent(sketchName);
        return this.$http.get(url, {
            cache: false
        });

    },
    resetGitWorkSpace: function() {

        var url = this.GIT_RESET_URL;
        var postData = {};

        return this.$http.post(url, postData, {
            cache: false
        });
    },
    validate: function(type, data) {
        var url = this.SKETCH_VALIDATE_URL;
        var postData = {
            'validationType': type,
            'validationData': data
        };
        return this.$http.post(url, postData, {
            cache: false
        });

    },
    getValidationDetails: function(key) {
        var url = this.SKETCH_VALIDATION_URL;
        var validationKey = key || null;
        if (validationKey) {
            url += key;
        }
        return this.$http.get(url, {
            cache: false
        });
    },

    getPromiseNotKeptLog: function(hostkey,sketchname,identifier) {

        var url = this.PROMISE_NOT_KEPT_URL + hostkey + '/' + sketchname +'/'+identifier;
        return this.$http.get(url, {
            cache: false
        });
    }
});



(function() {

    var sketchServiceProvider = Class.extend({
        instance: new sketchService(),
        $get: ['$http', function($http) {
            this.instance.$http = $http;
            return this.instance;
        }]
    });

    angular.module('dc.sketchService', [])
        .provider('sketchService', sketchServiceProvider);
}());