\set ON_ERROR_STOP true

-------------------------------------------------------------------------------
-- Create settings database:
-------------------------------------------------------------------------------

CREATE TABLE IF NOT EXISTS ScheduledReports (
	username		TEXT		NOT NULL,
	query			TEXT		NOT NULL,
	query_id		TEXT		NOT NULL,
	run_classes		TEXT		NOT NULL,
	last_executed 		TEXT,
	email			TEXT,
	email_title		TEXT,
	email_description	TEXT,
	host_include		TEXT[],
	host_exclude		TEXT[],
        already_run		BOOLEAN		DEFAULT FALSE,
        enabled			BOOLEAN		DEFAULT TRUE,
	output			TEXT[]		NOT NULL
    );

ALTER TABLE ScheduledReports
ADD COLUMN IF NOT EXISTS excludedhosts json DEFAULT NULL;

CREATE TABLE IF NOT EXISTS Settings (
        key                     TEXT		NOT NULL,
        value			JSON
    );
CREATE UNIQUE INDEX IF NOT EXISTS settings_unique_key_idx ON settings (key);

CREATE TABLE IF NOT EXISTS Users (
        username                TEXT		NOT NULL,
        password                TEXT,
        salt                    TEXT,
        name                    TEXT,
        email                   TEXT,
        external                BOOLEAN         DEFAULT FALSE,
        active                  BOOLEAN         DEFAULT FALSE,
        roles                   TEXT[]          DEFAULT '{}',
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

CREATE UNIQUE INDEX IF NOT EXISTS users_uniq ON Users (username, external);

CREATE TABLE IF NOT EXISTS Roles (
        name                    TEXT		NOT NULL,
        description             TEXT,
        include_rx              TEXT,
        exclude_rx              TEXT,
        sketches                TEXT[],
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

ALTER TABLE Roles ADD COLUMN IF NOT EXISTS is_default boolean NULL DEFAULT 'false';

DROP INDEX IF EXISTS roles_uniq;
CREATE UNIQUE INDEX IF NOT EXISTS roles_uniq ON Roles (name);

CREATE TABLE IF NOT EXISTS LicenseInfo (
        ExpireTimeStamp         timestamp with time zone    NOT NULL,
        InstallTimestamp        timestamp with time zone,
        Organization            TEXT,
        LicenseType             TEXT,
        LicenseCount            integer
    );

CREATE TABLE IF NOT EXISTS KeysPendingForDeletion (
        HostKey     TEXT
    );


 -- FOR OAUTH

CREATE TABLE IF NOT EXISTS  oauth_clients (client_id VARCHAR(80) NOT NULL, client_secret VARCHAR(80) NOT NULL, redirect_uri VARCHAR(2000) NOT NULL, grant_types VARCHAR(80), scope VARCHAR(100), user_id VARCHAR(80), CONSTRAINT client_id_pk PRIMARY KEY (client_id));
CREATE TABLE IF NOT EXISTS  oauth_access_tokens (access_token VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT access_token_pk PRIMARY KEY (access_token));
CREATE TABLE IF NOT EXISTS  oauth_authorization_codes (authorization_code VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), redirect_uri VARCHAR(2000), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT auth_code_pk PRIMARY KEY (authorization_code));
CREATE TABLE IF NOT EXISTS  oauth_refresh_tokens (refresh_token VARCHAR(40) NOT NULL, client_id VARCHAR(80) NOT NULL, user_id VARCHAR(255), expires TIMESTAMP NOT NULL, scope VARCHAR(2000), CONSTRAINT refresh_token_pk PRIMARY KEY (refresh_token));
--CREATE TABLE IF NOT EXISTS  oauth_users (username VARCHAR(255) NOT NULL, password VARCHAR(2000), first_name VARCHAR(255), last_name VARCHAR(255), CONSTRAINT username_pk PRIMARY KEY (username));
CREATE TABLE IF NOT EXISTS  oauth_scopes (scope TEXT, is_default BOOLEAN);
CREATE TABLE IF NOT EXISTS  oauth_jwt (client_id VARCHAR(80) NOT NULL, subject VARCHAR(80), public_key VARCHAR(2000), CONSTRAINT client_id_pk_jwt PRIMARY KEY (client_id));




CREATE OR REPLACE FUNCTION delete_expired_access() RETURNS TRIGGER AS $_$
BEGIN
    DELETE FROM oauth_access_tokens WHERE expires <  now() AT TIME ZONE 'UTC';
    DELETE FROM oauth_refresh_tokens WHERE expires < now() AT TIME ZONE 'UTC';
    RETURN NULL;
END $_$ LANGUAGE 'plpgsql';

DROP TRIGGER IF EXISTS delete_expired_access_token ON oauth_access_tokens;
CREATE TRIGGER delete_expired_access_token AFTER INSERT ON oauth_access_tokens FOR EACH ROW EXECUTE PROCEDURE delete_expired_access();

CREATE TABLE IF NOT EXISTS external_roles_map (
        external_role                    TEXT		NOT NULL,
        internal_role                TEXT,
        ChangeTimestamp         timestamp with time zone       DEFAULT now()
    );

CREATE UNIQUE INDEX IF NOT EXISTS external_roles_uniq ON external_roles_map (external_role);
